/*******************************************************************************
* E.S.O. - VLT project
*
* 
*
* who       when        what
* --------  ----------  ----------------------------------------------
* bgilli  yyyy-mm-dd  created from generic slalib sources
*/

/************************************************************************
*   NAME
* 
*   SYNOPSIS
* 
*   DESCRIPTION
*
*   FILES
*
*   ENVIRONMENT
*
*   RETURN VALUES 
*
*   CAUTIONS 
*
*   EXAMPLES
*
*   SEE ALSO
*
*   BUGS   
*
*------------------------------------------------------------------------
*/

#define _POSIX_SOURCE 1
#include "vltPort.h"

ATTRIBUTE_UNUSED static char *rcsId="@(#) $Id: nu00a.c,v 1.23 2009/07/30 00:04:13 vltsccm Exp $"; 

#include "slalib.h"
#include "slamac.h"
void slaNu00a ( double date, double *dpsi, double *deps )
/*
**  - - - - - - - - -
**   s l a N u 0 0 a
**  - - - - - - - - -
**
**  Nutation, IAU 2000A model (MHB_2000 luni-solar and planetary nutation
**  with free core nutation omitted).
**
**  Given:
**     date        double    TDB Modified Julian Date (Note 1)
**
**  Returned:
**     dpsi,deps   double*   nutation, luni-solar + planetary (Note 2)
**
**  Notes:
**
**  1) The date is TDB as an MJD (=JD-2400000.5).  TT will do.
**
**  2) The nutation components in longitude and obliquity are with
**     respect to the equinox and ecliptic of date.  The obliquity at
**     J2000 is assumed to be the Lieske et al. (1977) value of
**     84381.448 arcsec.
**
**  3) Both the luni-solar and planetary nutations are included.  The
**     latter are due to direct planetary nutations and the
**     perturbations of the lunar and terrestrial orbits.
**
**  4) The function computes the MHB_2000 nutation series with the
**     associated corrections for planetary nutations.  It is an
**     implementation of the nutation part of the IAU 2000A precession-
**     nutation model, formally adopted by the IAU General Assembly in
**     2000, namely MHB2000 (Mathews et al. 2002), but with the free
**     core nutation (FCN - see Note 5) omitted.
**
**  5) The full MHB_2000 model also contains contributions to the
**     nutations in longitude and obliquity due to the free-excitation
**     of the free-core-nutation during the period 1979-2000.  These FCN
**     terms, which are time-dependent and unpredictable, are NOT
**     included in the present function and, if required, must be
**     independently computed.  With the FCN corrections included, the
**     present function delivers a pole which is at current epochs
**     accurate to a few hundred microarcseconds.  The omission of FCN
**     introduces further errors of about that size.
**
**  6) The present function provides classical nutation.  The MHB_2000
**     algorithm, from which it is adapted, deals also with (i) the
**     offsets between the ICRS and mean poles and (ii) the adjustments
**     in longitude and obliquity due to the changed precession rates
**     with respect to IAU 1976 precession.
**
**  References:
**
**     Chapront, J., Chapront-Touze, M. & Francou, G. 2002,
**     Astron.Astrophys. 387, 700
**
**     Lieske, J.H., Lederle, T., Fricke, W. & Morando, B. 1977,
**     Astron.Astrophys. 58, 1-16
**
**     Mathews, P.M., Herring, T.A., Buffet, B.A. 2002, J.Geophys.Res.
**     107, B4.  The MHB_2000 code itself was obtained on 9th September
**     2002 from ftp//maia.usno.navy.mil/conv2000/chapter5/IAU2000A.
**
**     Simon, J.-L., Bretagnon, P., Chapront, J., Chapront-Touze, M.,
**     Francou, G., Laskar, J. 1994, Astron.Astrophys. 282, 663-683
**
**     Souchay, J., Loysel, B., Kinoshita, H., Folgueira, M. 1999,
**     Astron.Astrophys.Supp.Ser. 135, 111
**
**  Defined in slamac.h:  DAS2R, D2PI
**
**  Last revision:   22 October 2006
**
**  Copyright P.T.Wallace.  All rights reserved.
**
*/

#define DJC 36525.0            /* Days per Julian century */
#define DJM0 51544.5           /* Reference epoch (J2000), MJD */
#define TURNAS 1296000.0       /* Arc seconds in a full circle */

{

/* Fundamental arguments: l,l',F,D,Om,LMe,LVe,LE,LMa,LJu,LSa,LU,LN,pA */
#define NFA 14
   double fa[NFA];

/* ------------------------------ */
/* The luni-solar nutation series */
/* ------------------------------ */

   static struct {
      int nfa[5];    /* Amount of each fundamental argument */
      double ps;     /* Amplitude of sine term in longitude */
      double pst;    /* Amplitude of sine*t term in longitude (uas) */
      double pc;     /* Amplitude of cosine term in longitude (uas) */
      double ec;     /* Amplitude of cosine term in obliquity (uas) */
      double ect;    /* Amplitude of cosine*t term in obliquity (uas) */
      double es;     /* Amplitude of sine term in obliquity (uas) */
   } serls[] = {
/* Luni-solar terms 1-10 */
{{ 0,  0,  0,  0,  1},-17206416.1,-17466.6, 3338.6,9205233.1, 908.6,1537.7},
{{ 0,  0,  2, -2,  2}, -1317090.6,  -167.5,-1369.6, 573033.6,-301.5,-458.7},
{{ 0,  0,  2,  0,  2},  -227641.3,   -23.4,  279.6,  97845.9, -48.5, 137.4},
{{ 0,  0,  0,  0,  2},   207455.4,    20.7,  -69.8, -89749.2,  47.0, -29.1},
{{ 0,  1,  0,  0,  0},   147587.7,  -363.3, 1181.7,   7387.1, -18.4,-192.4},
{{ 0,  1,  2, -2,  2},   -51682.1,   122.6,  -52.4,  22438.6, -67.7, -17.4},
{{ 1,  0,  0,  0,  0},    71115.9,     7.3,  -87.2,   -675.0,   0.0,  35.8},
{{ 0,  0,  2,  0,  1},   -38729.8,   -36.7,   38.0,  20072.8,   1.8,  31.8},
{{ 1,  0,  2,  0,  2},   -30146.1,    -3.6,   81.6,  12902.5,  -6.3,  36.7},
{{ 0, -1,  2, -2,  2},    21582.9,   -49.4,   11.1,  -9592.9,  29.9,  13.2},
/* Luni-solar terms 11-20 */
{{ 0,  0,  2, -2,  1},    12822.7,    13.7,   18.1,  -6898.2,  -0.9,   3.9},
{{-1,  0,  2,  0,  2},    12345.7,     1.1,    1.9,  -5331.1,   3.2,  -0.4},
{{-1,  0,  0,  2,  0},    15699.4,     1.0,  -16.8,   -123.5,   0.0,   8.2},
{{ 1,  0,  0,  0,  1},     6311.0,     6.3,    2.7,  -3322.8,   0.0,  -0.9},
{{-1,  0,  0,  0,  1},    -5797.6,    -6.3,  -18.9,   3142.9,   0.0,  -7.5},
{{-1,  0,  2,  2,  2},    -5964.1,    -1.1,   14.9,   2554.3,  -1.1,   6.6},
{{ 1,  0,  2,  0,  1},    -5161.3,    -4.2,   12.9,   2636.6,   0.0,   7.8},
{{-2,  0,  2,  0,  1},     4589.3,     5.0,    3.1,  -2423.6,  -1.0,   2.0},
{{ 0,  0,  0,  2,  0},     6338.4,     1.1,  -15.0,   -122.0,   0.0,   2.9},
{{ 0,  0,  2,  2,  2},    -3857.1,    -0.1,   15.8,   1645.2,  -1.1,   6.8},
/* Luni-solar terms 21-30 */
{{ 0, -2,  2, -2,  2},     3248.1,     0.0,    0.0,  -1387.0,   0.0,   0.0},
{{-2,  0,  0,  2,  0},    -4772.2,     0.0,   -1.8,     47.7,   0.0,  -2.5},
{{ 2,  0,  2,  0,  2},    -3104.6,    -0.1,   13.1,   1323.8,  -1.1,   5.9},
{{ 1,  0,  2, -2,  2},     2859.3,     0.0,   -0.1,  -1233.8,   1.0,  -0.3},
{{-1,  0,  2,  0,  1},     2044.1,     2.1,    1.0,  -1075.8,   0.0,  -0.3},
{{ 2,  0,  0,  0,  0},     2924.3,     0.0,   -7.4,    -60.9,   0.0,   1.3},
{{ 0,  0,  2,  0,  0},     2588.7,     0.0,   -6.6,    -55.0,   0.0,   1.1},
{{ 0,  1,  0,  0,  1},    -1405.3,    -2.5,    7.9,    855.1,  -0.2,  -4.5},
{{-1,  0,  0,  2,  1},     1516.4,     1.0,    1.1,   -800.1,   0.0,  -0.1},
{{ 0,  2,  2, -2,  2},    -1579.4,     7.2,   -1.6,    685.0,  -4.2,  -0.5},
/* Luni-solar terms 31-40 */
{{ 0,  0, -2,  2,  0},     2178.3,     0.0,    1.3,    -16.7,   0.0,   1.3},
{{ 1,  0,  0, -2,  1},    -1287.3,    -1.0,   -3.7,    695.3,   0.0,  -1.4},
{{ 0, -1,  0,  0,  1},    -1265.4,     1.1,    6.3,    641.5,   0.0,   2.6},
{{-1,  0,  2,  2,  1},    -1020.4,     0.0,    2.5,    522.2,   0.0,   1.5},
{{ 0,  2,  0,  0,  0},     1670.7,    -8.5,   -1.0,     16.8,  -0.1,   1.0},
{{ 1,  0,  2,  2,  2},     -769.1,     0.0,    4.4,    326.8,   0.0,   1.9},
{{-2,  0,  2,  0,  0},    -1102.4,     0.0,   -1.4,     10.4,   0.0,   0.2},
{{ 0,  1,  2,  0,  2},      756.6,    -2.1,   -1.1,   -325.0,   0.0,  -0.5},
{{ 0,  0,  2,  2,  1},     -663.7,    -1.1,    2.5,    335.3,   0.0,   1.4},
{{ 0, -1,  2,  0,  2},     -714.1,     2.1,    0.8,    307.0,   0.0,   0.4},
/* Luni-solar terms 41-50 */
{{ 0,  0,  0,  2,  1},     -630.2,    -1.1,    0.2,    327.2,   0.0,   0.4},
{{ 1,  0,  2, -2,  1},      580.0,     1.0,    0.2,   -304.5,   0.0,  -0.1},
{{ 2,  0,  2, -2,  2},      644.3,     0.0,   -0.7,   -276.8,   0.0,  -0.4},
{{-2,  0,  0,  2,  1},     -577.4,    -1.1,   -1.5,    304.1,   0.0,  -0.5},
{{ 2,  0,  2,  0,  1},     -535.0,     0.0,    2.1,    269.5,   0.0,   1.2},
{{ 0, -1,  2, -2,  1},     -475.2,    -1.1,   -0.3,    271.9,   0.0,  -0.3},
{{ 0,  0,  0, -2,  1},     -494.0,    -1.1,   -2.1,    272.0,   0.0,  -0.9},
{{-1, -1,  0,  2,  0},      735.0,     0.0,   -0.8,     -5.1,   0.0,   0.4},
{{ 2,  0,  0, -2,  1},      406.5,     0.0,    0.6,   -220.6,   0.0,   0.1},
{{ 1,  0,  0,  2,  0},      657.9,     0.0,   -2.4,    -19.9,   0.0,   0.2},
/* Luni-solar terms 51-60 */
{{ 0,  1,  2, -2,  1},      357.9,     0.0,    0.5,   -190.0,   0.0,   0.1},
{{ 1, -1,  0,  0,  0},      472.5,     0.0,   -0.6,     -4.1,   0.0,   0.3},
{{-2,  0,  2,  0,  2},     -307.5,     0.0,   -0.2,    131.3,   0.0,  -0.1},
{{ 3,  0,  2,  0,  2},     -290.4,     0.0,    1.5,    123.3,   0.0,   0.7},
{{ 0, -1,  0,  2,  0},      434.8,     0.0,   -1.0,     -8.1,   0.0,   0.2},
{{ 1, -1,  2,  0,  2},     -287.8,     0.0,    0.8,    123.2,   0.0,   0.4},
{{ 0,  0,  0,  1,  0},     -423.0,     0.0,    0.5,     -2.0,   0.0,  -0.2},
{{-1, -1,  2,  2,  2},     -281.9,     0.0,    0.7,    120.7,   0.0,   0.3},
{{-1,  0,  2,  0,  0},     -405.6,     0.0,    0.5,      4.0,   0.0,  -0.2},
{{ 0, -1,  2,  2,  2},     -264.7,     0.0,    1.1,    112.9,   0.0,   0.5},
/* Luni-solar terms 61-70 */
{{-2,  0,  0,  0,  1},     -229.4,     0.0,   -1.0,    126.6,   0.0,  -0.4},
{{ 1,  1,  2,  0,  2},      248.1,     0.0,   -0.7,   -106.2,   0.0,  -0.3},
{{ 2,  0,  0,  0,  1},      217.9,     0.0,   -0.2,   -112.9,   0.0,  -0.2},
{{-1,  1,  0,  1,  0},      327.6,     0.0,    0.1,     -0.9,   0.0,   0.0},
{{ 1,  1,  0,  0,  0},     -338.9,     0.0,    0.5,      3.5,   0.0,  -0.2},
{{ 1,  0,  2,  0,  0},      333.9,     0.0,   -1.3,    -10.7,   0.0,   0.1},
{{-1,  0,  2, -2,  1},     -198.7,     0.0,   -0.6,    107.3,   0.0,  -0.2},
{{ 1,  0,  0,  0,  2},     -198.1,     0.0,    0.0,     85.4,   0.0,   0.0},
{{-1,  0,  0,  1,  0},      402.6,     0.0,  -35.3,    -55.3,   0.0, -13.9},
{{ 0,  0,  2,  1,  2},      166.0,     0.0,   -0.5,    -71.0,   0.0,  -0.2},
/* Luni-solar terms 71-80 */
{{-1,  0,  2,  4,  2},     -152.1,     0.0,    0.9,     64.7,   0.0,   0.4},
{{-1,  1,  0,  1,  1},      131.4,     0.0,    0.0,    -70.0,   0.0,   0.0},
{{ 0, -2,  2, -2,  1},     -128.3,     0.0,    0.0,     67.2,   0.0,   0.0},
{{ 1,  0,  2,  2,  1},     -133.1,     0.0,    0.8,     66.3,   0.0,   0.4},
{{-2,  0,  2,  2,  2},      138.3,     0.0,   -0.2,    -59.4,   0.0,  -0.2},
{{-1,  0,  0,  0,  2},      140.5,     0.0,    0.4,    -61.0,   0.0,   0.2},
{{ 1,  1,  2, -2,  2},      129.0,     0.0,    0.0,    -55.6,   0.0,   0.0},
{{-2,  0,  2,  4,  2},     -121.4,     0.0,    0.5,     51.8,   0.0,   0.2},
{{-1,  0,  4,  0,  2},      114.6,     0.0,   -0.3,    -49.0,   0.0,  -0.1},
{{ 2,  0,  2, -2,  1},      101.9,     0.0,   -0.1,    -52.7,   0.0,  -0.1},
/* Luni-solar terms 81-90 */
{{ 2,  0,  2,  2,  2},     -110.0,     0.0,    0.9,     46.5,   0.0,   0.4},
{{ 1,  0,  0,  2,  1},      -97.0,     0.0,    0.2,     49.6,   0.0,   0.1},
{{ 3,  0,  0,  0,  0},      157.5,     0.0,   -0.6,     -5.0,   0.0,   0.0},
{{ 3,  0,  2, -2,  2},       93.4,     0.0,   -0.3,    -39.9,   0.0,  -0.1},
{{ 0,  0,  4, -2,  2},       92.2,     0.0,   -0.1,    -39.5,   0.0,  -0.1},
{{ 0,  1,  2,  0,  1},       81.5,     0.0,   -0.1,    -42.2,   0.0,  -0.1},
{{ 0,  0, -2,  2,  1},       83.4,     0.0,    0.2,    -44.0,   0.0,   0.1},
{{ 0,  0,  2, -2,  3},      124.8,     0.0,    0.0,    -17.0,   0.0,   0.1},
{{-1,  0,  0,  4,  0},      133.8,     0.0,   -0.5,     -3.9,   0.0,   0.0},
{{ 2,  0, -2,  0,  1},       71.6,     0.0,   -0.2,    -38.9,   0.0,  -0.1},
/* Luni-solar terms 91-100 */
{{-2,  0,  0,  4,  0},      128.2,     0.0,   -0.3,     -2.3,   0.0,   0.1},
{{-1, -1,  0,  2,  1},       74.2,     0.0,    0.1,    -39.1,   0.0,   0.0},
{{-1,  0,  0,  1,  1},      102.0,     0.0,   -2.5,    -49.5,   0.0,  -1.0},
{{ 0,  1,  0,  0,  2},       71.5,     0.0,   -0.4,    -32.6,   0.0,   0.2},
{{ 0,  0, -2,  0,  1},      -66.6,     0.0,   -0.3,     36.9,   0.0,  -0.1},
{{ 0, -1,  2,  0,  1},      -66.7,     0.0,    0.1,     34.6,   0.0,   0.1},
{{ 0,  0,  2, -1,  2},      -70.4,     0.0,    0.0,     30.4,   0.0,   0.0},
{{ 0,  0,  2,  4,  2},      -69.4,     0.0,    0.5,     29.4,   0.0,   0.2},
{{-2, -1,  0,  2,  0},     -101.4,     0.0,   -0.1,      0.4,   0.0,  -0.1},
{{ 1,  1,  0, -2,  1},      -58.5,     0.0,   -0.2,     31.6,   0.0,  -0.1},
/* Luni-solar terms 101-110 */
{{-1,  1,  0,  2,  0},      -94.9,     0.0,    0.1,      0.8,   0.0,  -0.1},
{{-1,  1,  0,  1,  2},      -59.5,     0.0,    0.0,     25.8,   0.0,   0.0},
{{ 1, -1,  0,  0,  1},       52.8,     0.0,    0.0,    -27.9,   0.0,   0.0},
{{ 1, -1,  2,  2,  2},      -59.0,     0.0,    0.4,     25.2,   0.0,   0.2},
{{-1,  1,  2,  2,  2},       57.0,     0.0,   -0.2,    -24.4,   0.0,  -0.1},
{{ 3,  0,  2,  0,  1},      -50.2,     0.0,    0.3,     25.0,   0.0,   0.2},
{{ 0,  1, -2,  2,  0},      -87.5,     0.0,    0.1,      2.9,   0.0,   0.0},
{{-1,  0,  0, -2,  1},      -49.2,     0.0,   -0.3,     27.5,   0.0,  -0.1},
{{ 0,  1,  2,  2,  2},       53.5,     0.0,   -0.2,    -22.8,   0.0,  -0.1},
{{-1, -1,  2,  2,  1},      -46.7,     0.0,    0.1,     24.0,   0.0,   0.1},
/* Luni-solar terms 111-120 */
{{ 0, -1,  0,  0,  2},       59.1,     0.0,    0.0,    -25.3,   0.0,   0.0},
{{ 1,  0,  2, -4,  1},      -45.3,     0.0,   -0.1,     24.4,   0.0,  -0.1},
{{-1,  0, -2,  2,  0},       76.6,     0.0,    0.1,      0.9,   0.0,   0.0},
{{ 0, -1,  2,  2,  1},      -44.6,     0.0,    0.2,     22.5,   0.0,   0.1},
{{ 2, -1,  2,  0,  2},      -48.8,     0.0,    0.2,     20.7,   0.0,   0.1},
{{ 0,  0,  0,  2,  2},      -46.8,     0.0,    0.0,     20.1,   0.0,   0.0},
{{ 1, -1,  2,  0,  1},      -42.1,     0.0,    0.1,     21.6,   0.0,   0.1},
{{-1,  1,  2,  0,  2},       46.3,     0.0,    0.0,    -20.0,   0.0,   0.0},
{{ 0,  1,  0,  2,  0},      -67.3,     0.0,    0.2,      1.4,   0.0,   0.0},
{{ 0, -1, -2,  2,  0},       65.8,     0.0,    0.0,     -0.2,   0.0,   0.0},
/* Luni-solar terms 121-130 */
{{ 0,  3,  2, -2,  2},      -43.8,     0.0,    0.0,     18.8,   0.0,   0.0},
{{ 0,  0,  0,  1,  1},      -39.0,     0.0,    0.0,     20.5,   0.0,   0.0},
{{-1,  0,  2,  2,  0},       63.9,    -1.1,   -0.2,     -1.9,   0.0,   0.0},
{{ 2,  1,  2,  0,  2},       41.2,     0.0,   -0.2,    -17.6,   0.0,  -0.1},
{{ 1,  1,  0,  0,  1},      -36.1,     0.0,    0.0,     18.9,   0.0,   0.0},
{{ 1,  1,  2,  0,  1},       36.0,     0.0,   -0.1,    -18.5,   0.0,  -0.1},
{{ 2,  0,  0,  2,  0},       58.8,     0.0,   -0.3,     -2.4,   0.0,   0.0},
{{ 1,  0, -2,  2,  0},      -57.8,     0.0,    0.1,      0.5,   0.0,   0.0},
{{-1,  0,  0,  2,  2},      -39.6,     0.0,    0.0,     17.1,   0.0,   0.0},
{{ 0,  1,  0,  1,  0},       56.5,     0.0,   -0.1,     -0.6,   0.0,   0.0},
/* Luni-solar terms 131-140 */
{{ 0,  1,  0, -2,  1},      -33.5,     0.0,   -0.1,     18.4,   0.0,  -0.1},
{{-1,  0,  2, -2,  2},       35.7,     0.0,    0.1,    -15.4,   0.0,   0.0},
{{ 0,  0,  0, -1,  1},       32.1,     0.0,    0.1,    -17.4,   0.0,   0.0},
{{-1,  1,  0,  0,  1},      -30.1,     0.0,   -0.1,     16.2,   0.0,   0.0},
{{ 1,  0,  2, -1,  2},      -33.4,     0.0,    0.0,     14.4,   0.0,   0.0},
{{ 1, -1,  0,  2,  0},       49.3,     0.0,   -0.2,     -1.5,   0.0,   0.0},
{{ 0,  0,  0,  4,  0},       49.4,     0.0,   -0.2,     -1.9,   0.0,   0.0},
{{ 1,  0,  2,  1,  2},       33.7,     0.0,   -0.1,    -14.3,   0.0,  -0.1},
{{ 0,  0,  2,  1,  1},       28.0,     0.0,   -0.1,    -14.4,   0.0,   0.0},
{{ 1,  0,  0, -2,  2},       30.9,     0.0,    0.1,    -13.4,   0.0,   0.0},
/* Luni-solar terms 141-150 */
{{-1,  0,  2,  4,  1},      -26.3,     0.0,    0.2,     13.1,   0.0,   0.1},
{{ 1,  0, -2,  0,  1},       25.3,     0.0,    0.1,    -13.8,   0.0,   0.0},
{{ 1,  1,  2, -2,  1},       24.5,     0.0,    0.0,    -12.8,   0.0,   0.0},
{{ 0,  0,  2,  2,  0},       41.6,     0.0,   -0.2,     -1.7,   0.0,   0.0},
{{-1,  0,  2, -1,  1},      -22.9,     0.0,    0.0,     12.8,   0.0,   0.0},
{{-2,  0,  2,  2,  1},       23.1,     0.0,    0.0,    -12.0,   0.0,   0.0},
{{ 4,  0,  2,  0,  2},      -25.9,     0.0,    0.2,     10.9,   0.0,   0.1},
{{ 2, -1,  0,  0,  0},       37.5,     0.0,   -0.1,     -0.8,   0.0,   0.0},
{{ 2,  1,  2, -2,  2},       25.2,     0.0,    0.0,    -10.8,   0.0,   0.0},
{{ 0,  1,  2,  1,  2},      -24.5,     0.0,    0.1,     10.4,   0.0,   0.0},
/* Luni-solar terms 151-160 */
{{ 1,  0,  4, -2,  2},       24.3,     0.0,   -0.1,    -10.4,   0.0,   0.0},
{{-1, -1,  0,  0,  1},       20.8,     0.0,    0.1,    -11.2,   0.0,   0.0},
{{ 0,  1,  0,  2,  1},       19.9,     0.0,    0.0,    -10.2,   0.0,   0.0},
{{-2,  0,  2,  4,  1},      -20.8,     0.0,    0.1,     10.5,   0.0,   0.0},
{{ 2,  0,  2,  0,  0},       33.5,     0.0,   -0.2,     -1.4,   0.0,   0.0},
{{ 1,  0,  0,  1,  0},      -32.5,     0.0,    0.1,      0.7,   0.0,   0.0},
{{-1,  0,  0,  4,  1},      -18.7,     0.0,    0.0,      9.6,   0.0,   0.0},
{{-1,  0,  4,  0,  1},       19.7,     0.0,   -0.1,    -10.0,   0.0,   0.0},
{{ 2,  0,  2,  2,  1},      -19.2,     0.0,    0.2,      9.4,   0.0,   0.1},
{{ 0,  0,  2, -3,  2},      -18.8,     0.0,    0.0,      8.3,   0.0,   0.0},
/* Luni-solar terms 161-170 */
{{-1, -2,  0,  2,  0},       27.6,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 2,  1,  0,  0,  0},      -28.6,     0.0,    0.1,      0.6,   0.0,   0.0},
{{ 0,  0,  4,  0,  2},       18.6,     0.0,   -0.1,     -7.9,   0.0,   0.0},
{{ 0,  0,  0,  0,  3},      -21.9,     0.0,    0.0,      4.3,   0.0,   0.0},
{{ 0,  3,  0,  0,  0},       27.6,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 0,  0,  2, -4,  1},      -15.3,     0.0,   -0.1,      8.4,   0.0,   0.0},
{{ 0, -1,  0,  2,  1},      -15.6,     0.0,    0.0,      8.1,   0.0,   0.0},
{{ 0,  0,  0,  4,  1},      -15.4,     0.0,    0.1,      7.8,   0.0,   0.0},
{{-1, -1,  2,  4,  2},      -17.4,     0.0,    0.1,      7.5,   0.0,   0.0},
{{ 1,  0,  2,  4,  2},      -16.3,     0.0,    0.2,      6.9,   0.0,   0.1},
/* Luni-solar terms 171-180 */
{{-2,  2,  0,  2,  0},      -22.8,     0.0,    0.0,      0.1,   0.0,   0.0},
{{-2, -1,  2,  0,  1},        9.1,     0.0,   -0.4,     -5.4,   0.0,  -0.2},
{{-2,  0,  0,  2,  2},       17.5,     0.0,    0.0,     -7.5,   0.0,   0.0},
{{-1, -1,  2,  0,  2},      -15.9,     0.0,    0.0,      6.9,   0.0,   0.0},
{{ 0,  0,  4, -2,  1},       14.1,     0.0,    0.0,     -7.2,   0.0,   0.0},
{{ 3,  0,  2, -2,  1},       14.7,     0.0,    0.0,     -7.5,   0.0,   0.0},
{{-2, -1,  0,  2,  1},      -13.2,     0.0,    0.0,      6.9,   0.0,   0.0},
{{ 1,  0,  0, -1,  1},       15.9,     0.0,   -2.8,     -5.4,   0.0,   1.1},
{{ 0, -2,  0,  2,  0},       21.3,     0.0,    0.0,     -0.4,   0.0,   0.0},
{{-2,  0,  0,  4,  1},       12.3,     0.0,    0.0,     -6.4,   0.0,   0.0},
/* Luni-solar terms 181-190 */
{{-3,  0,  0,  0,  1},      -11.8,     0.0,   -0.1,      6.6,   0.0,   0.0},
{{ 1,  1,  2,  2,  2},       14.4,     0.0,   -0.1,     -6.1,   0.0,   0.0},
{{ 0,  0,  2,  4,  1},      -12.1,     0.0,    0.1,      6.0,   0.0,   0.0},
{{ 3,  0,  2,  2,  2},      -13.4,     0.0,    0.1,      5.6,   0.0,   0.1},
{{-1,  1,  2, -2,  1},      -10.5,     0.0,    0.0,      5.7,   0.0,   0.0},
{{ 2,  0,  0, -4,  1},      -10.2,     0.0,    0.0,      5.6,   0.0,   0.0},
{{ 0,  0,  0, -2,  2},       12.0,     0.0,    0.0,     -5.2,   0.0,   0.0},
{{ 2,  0,  2, -4,  1},       10.1,     0.0,    0.0,     -5.4,   0.0,   0.0},
{{-1,  1,  0,  2,  1},      -11.3,     0.0,    0.0,      5.9,   0.0,   0.0},
{{ 0,  0,  2, -1,  1},      -10.6,     0.0,    0.0,      6.1,   0.0,   0.0},
/* Luni-solar terms 191-200 */
{{ 0, -2,  2,  2,  2},      -12.9,     0.0,    0.1,      5.5,   0.0,   0.0},
{{ 2,  0,  0,  2,  1},      -11.4,     0.0,    0.0,      5.7,   0.0,   0.0},
{{ 4,  0,  2, -2,  2},       11.3,     0.0,   -0.1,     -4.9,   0.0,   0.0},
{{ 2,  0,  0, -2,  2},      -10.2,     0.0,    0.0,      4.4,   0.0,   0.0},
{{ 0,  2,  0,  0,  1},       -9.4,     0.0,    0.0,      5.1,   0.0,   0.0},
{{ 1,  0,  0, -4,  1},      -10.0,     0.0,   -0.1,      5.6,   0.0,   0.0},
{{ 0,  2,  2, -2,  1},        8.7,     0.0,    0.0,     -4.7,   0.0,   0.0},
{{-3,  0,  0,  4,  0},       16.1,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{-1,  1,  2,  0,  1},        9.6,     0.0,    0.0,     -5.0,   0.0,   0.0},
{{-1, -1,  0,  4,  0},       15.1,     0.0,   -0.1,     -0.5,   0.0,   0.0},
/* Luni-solar terms 201-210 */
{{-1, -2,  2,  2,  2},      -10.4,     0.0,    0.0,      4.4,   0.0,   0.0},
{{-2, -1,  2,  4,  2},      -11.0,     0.0,    0.0,      4.8,   0.0,   0.0},
{{ 1, -1,  2,  2,  1},      -10.0,     0.0,    0.1,      5.0,   0.0,   0.0},
{{-2,  1,  0,  2,  0},        9.2,     0.0,   -0.5,      1.2,   0.0,  -0.2},
{{-2,  1,  2,  0,  1},        8.2,     0.0,    0.0,     -4.5,   0.0,   0.0},
{{ 2,  1,  0, -2,  1},        8.2,     0.0,    0.0,     -4.5,   0.0,   0.0},
{{-3,  0,  2,  0,  1},       -7.8,     0.0,    0.0,      4.1,   0.0,   0.0},
{{-2,  0,  2, -2,  1},       -7.7,     0.0,    0.0,      4.3,   0.0,   0.0},
{{-1,  1,  0,  2,  2},        0.2,     0.0,    0.0,      5.4,   0.0,   0.0},
{{ 0, -1,  2, -1,  2},        9.4,     0.0,    0.0,     -4.0,   0.0,   0.0},
/* Luni-solar terms 211-220 */
{{-1,  0,  4, -2,  2},       -9.3,     0.0,    0.0,      4.0,   0.0,   0.0},
{{ 0, -2,  2,  0,  2},       -8.3,     0.0,    1.0,      4.0,   0.0,  -0.2},
{{-1,  0,  2,  1,  2},        8.3,     0.0,    0.0,     -3.6,   0.0,   0.0},
{{ 2,  0,  0,  0,  2},       -9.1,     0.0,    0.0,      3.9,   0.0,   0.0},
{{ 0,  0,  2,  0,  3},       12.8,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{-2,  0,  4,  0,  2},       -7.9,     0.0,    0.0,      3.4,   0.0,   0.0},
{{-1,  0, -2,  0,  1},       -8.3,     0.0,    0.0,      4.7,   0.0,   0.0},
{{-1,  1,  2,  2,  1},        8.4,     0.0,    0.0,     -4.4,   0.0,   0.0},
{{ 3,  0,  0,  0,  1},        8.3,     0.0,    0.0,     -4.3,   0.0,   0.0},
{{-1,  0,  2,  3,  2},        9.1,     0.0,    0.0,     -3.9,   0.0,   0.0},
/* Luni-solar terms 221-230 */
{{ 2, -1,  2,  0,  1},       -7.7,     0.0,    0.0,      3.9,   0.0,   0.0},
{{ 0,  1,  2,  2,  1},        8.4,     0.0,    0.0,     -4.3,   0.0,   0.0},
{{ 0, -1,  2,  4,  2},       -9.2,     0.0,    0.1,      3.9,   0.0,   0.0},
{{ 2, -1,  2,  2,  2},       -9.2,     0.0,    0.1,      3.9,   0.0,   0.0},
{{ 0,  2, -2,  2,  0},       -9.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1, -1,  2, -1,  1},        6.8,     0.0,    0.0,     -3.6,   0.0,   0.0},
{{ 0, -2,  0,  0,  1},       -6.1,     0.0,    0.0,      3.2,   0.0,   0.0},
{{ 1,  0,  2, -4,  2},        7.1,     0.0,    0.0,     -3.1,   0.0,   0.0},
{{ 1, -1,  0, -2,  1},        6.2,     0.0,    0.0,     -3.4,   0.0,   0.0},
{{-1, -1,  2,  0,  1},       -6.3,     0.0,    0.0,      3.3,   0.0,   0.0},
/* Luni-solar terms 231-240 */
{{ 1, -1,  2, -2,  2},       -7.3,     0.0,    0.0,      3.2,   0.0,   0.0},
{{-2, -1,  0,  4,  0},       11.5,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{-1,  0,  0,  3,  0},      -10.3,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-2, -1,  2,  2,  2},        6.3,     0.0,    0.0,     -2.8,   0.0,   0.0},
{{ 0,  2,  2,  0,  2},        7.4,     0.0,    0.0,     -3.2,   0.0,   0.0},
{{ 1,  1,  0,  2,  0},      -10.3,     0.0,   -0.3,      0.3,   0.0,  -0.1},
{{ 2,  0,  2, -1,  2},       -6.9,     0.0,    0.0,      3.0,   0.0,   0.0},
{{ 1,  0,  2,  1,  1},        5.7,     0.0,    0.0,     -2.9,   0.0,   0.0},
{{ 4,  0,  0,  0,  0},        9.4,     0.0,    0.0,     -0.4,   0.0,   0.0},
{{ 2,  1,  2,  0,  1},        6.4,     0.0,    0.0,     -3.3,   0.0,   0.0},
/* Luni-solar terms 241-250 */
{{ 3, -1,  2,  0,  2},       -6.3,     0.0,    0.0,      2.6,   0.0,   0.0},
{{-2,  2,  0,  2,  1},       -3.8,     0.0,    0.0,      2.0,   0.0,   0.0},
{{ 1,  0,  2, -3,  1},       -4.3,     0.0,    0.0,      2.4,   0.0,   0.0},
{{ 1,  1,  2, -4,  1},       -4.5,     0.0,    0.0,      2.3,   0.0,   0.0},
{{-1, -1,  2, -2,  1},        4.7,     0.0,    0.0,     -2.4,   0.0,   0.0},
{{ 0, -1,  0, -1,  1},       -4.8,     0.0,    0.0,      2.5,   0.0,   0.0},
{{ 0, -1,  0, -2,  1},        4.5,     0.0,    0.0,     -2.6,   0.0,   0.0},
{{-2,  0,  0,  0,  2},        5.6,     0.0,    0.0,     -2.5,   0.0,   0.0},
{{-2,  0, -2,  2,  0},        8.8,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-1,  0, -2,  4,  0},       -7.5,     0.0,    0.0,      0.0,   0.0,   0.0},
/* Luni-solar terms 251-260 */
{{ 1, -2,  0,  0,  0},        8.5,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  1,  0,  1,  1},        4.9,     0.0,    0.0,     -2.6,   0.0,   0.0},
{{-1,  2,  0,  2,  0},       -7.4,     0.0,   -0.3,     -0.1,   0.0,  -0.1},
{{ 1, -1,  2, -2,  1},       -3.9,     0.0,    0.0,      2.1,   0.0,   0.0},
{{ 1,  2,  2, -2,  2},        4.5,     0.0,    0.0,     -2.0,   0.0,   0.0},
{{ 2, -1,  2, -2,  2},        5.1,     0.0,    0.0,     -2.2,   0.0,   0.0},
{{ 1,  0,  2, -1,  1},       -4.0,     0.0,    0.0,      2.1,   0.0,   0.0},
{{ 2,  1,  2, -2,  1},        4.1,     0.0,    0.0,     -2.1,   0.0,   0.0},
{{-2,  0,  0, -2,  1},       -4.2,     0.0,    0.0,      2.4,   0.0,   0.0},
{{ 1, -2,  2,  0,  2},       -5.1,     0.0,    0.0,      2.2,   0.0,   0.0},
/* Luni-solar terms 261-270 */
{{ 0,  1,  2,  1,  1},       -4.2,     0.0,    0.0,      2.2,   0.0,   0.0},
{{ 1,  0,  4, -2,  1},        3.9,     0.0,    0.0,     -2.1,   0.0,   0.0},
{{-2,  0,  4,  2,  2},        4.6,     0.0,    0.0,     -1.8,   0.0,   0.0},
{{ 1,  1,  2,  1,  2},       -5.3,     0.0,    0.0,      2.2,   0.0,   0.0},
{{ 1,  0,  0,  4,  0},        8.2,     0.0,    0.0,     -0.4,   0.0,   0.0},
{{ 1,  0,  2,  2,  0},        8.1,     0.0,   -0.1,     -0.4,   0.0,   0.0},
{{ 2,  0,  2,  1,  2},        4.7,     0.0,    0.0,     -1.9,   0.0,   0.0},
{{ 3,  1,  2,  0,  2},        5.3,     0.0,    0.0,     -2.3,   0.0,   0.0},
{{ 4,  0,  2,  0,  1},       -4.5,     0.0,    0.0,      2.2,   0.0,   0.0},
{{-2, -1,  2,  0,  0},       -4.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
/* Luni-solar terms 271-280 */
{{ 0,  1, -2,  2,  1},       -3.3,     0.0,    0.0,      1.6,   0.0,   0.0},
{{ 1,  0, -2,  1,  0},       -6.1,     0.0,    0.0,      0.1,   0.0,   0.0},
{{ 0, -1, -2,  2,  1},        2.8,     0.0,    0.0,     -1.5,   0.0,   0.0},
{{ 2, -1,  0, -2,  1},       -3.8,     0.0,    0.0,      1.9,   0.0,   0.0},
{{-1,  0,  2, -1,  2},       -3.3,     0.0,    0.0,      2.1,   0.0,   0.0},
{{ 1,  0,  2, -3,  2},       -6.0,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  1,  2, -2,  3},        4.8,     0.0,    0.0,     -1.0,   0.0,   0.0},
{{ 0,  0,  2, -3,  1},        2.7,     0.0,    0.0,     -1.4,   0.0,   0.0},
{{-1,  0, -2,  2,  1},        3.8,     0.0,    0.0,     -2.0,   0.0,   0.0},
{{ 0,  0,  2, -4,  2},        3.1,     0.0,    0.0,     -1.3,   0.0,   0.0},
/* Luni-solar terms 281-290 */
{{-2,  1,  0,  0,  1},       -2.9,     0.0,    0.0,      1.5,   0.0,   0.0},
{{-1,  0,  0, -1,  1},        2.8,     0.0,    0.0,     -1.5,   0.0,   0.0},
{{ 2,  0,  2, -4,  2},       -3.2,     0.0,    0.0,      1.5,   0.0,   0.0},
{{ 0,  0,  4, -4,  4},        4.5,     0.0,    0.0,     -0.8,   0.0,   0.0},
{{ 0,  0,  4, -4,  2},       -4.4,     0.0,    0.0,      1.9,   0.0,   0.0},
{{-1, -2,  0,  2,  1},        2.8,     0.0,    0.0,     -1.5,   0.0,   0.0},
{{-2,  0,  0,  3,  0},       -5.1,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1,  0, -2,  2,  1},       -3.6,     0.0,    0.0,      2.0,   0.0,   0.0},
{{-3,  0,  2,  2,  2},        4.4,     0.0,    0.0,     -1.9,   0.0,   0.0},
{{-3,  0,  2,  2,  1},        2.6,     0.0,    0.0,     -1.4,   0.0,   0.0},
/* Luni-solar terms 291-300 */
{{-2,  0,  2,  2,  0},       -6.0,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 2, -1,  0,  0,  1},        3.5,     0.0,    0.0,     -1.8,   0.0,   0.0},
{{-2,  1,  2,  2,  2},       -2.7,     0.0,    0.0,      1.1,   0.0,   0.0},
{{ 1,  1,  0,  1,  0},        4.7,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 0,  1,  4, -2,  2},        3.6,     0.0,    0.0,     -1.5,   0.0,   0.0},
{{-1,  1,  0, -2,  1},       -3.6,     0.0,    0.0,      2.0,   0.0,   0.0},
{{ 0,  0,  0, -4,  1},       -3.5,     0.0,    0.0,      1.9,   0.0,   0.0},
{{ 1, -1,  0,  2,  1},       -3.7,     0.0,    0.0,      1.9,   0.0,   0.0},
{{ 1,  1,  0,  2,  1},        3.2,     0.0,    0.0,     -1.6,   0.0,   0.0},
{{-1,  2,  2,  2,  2},        3.5,     0.0,    0.0,     -1.4,   0.0,   0.0},
/* Luni-solar terms 301-310 */
{{ 3,  1,  2, -2,  2},        3.2,     0.0,    0.0,     -1.3,   0.0,   0.0},
{{ 0, -1,  0,  4,  0},        6.5,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 2, -1,  0,  2,  0},        4.7,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 0,  0,  4,  0,  1},        3.2,     0.0,    0.0,     -1.6,   0.0,   0.0},
{{ 2,  0,  4, -2,  2},        3.7,     0.0,    0.0,     -1.6,   0.0,   0.0},
{{-1, -1,  2,  4,  1},       -3.0,     0.0,    0.0,      1.5,   0.0,   0.0},
{{ 1,  0,  0,  4,  1},       -3.2,     0.0,    0.0,      1.6,   0.0,   0.0},
{{ 1, -2,  2,  2,  2},       -3.1,     0.0,    0.0,      1.3,   0.0,   0.0},
{{ 0,  0,  2,  3,  2},        3.7,     0.0,    0.0,     -1.6,   0.0,   0.0},
{{-1,  1,  2,  4,  2},        3.1,     0.0,    0.0,     -1.3,   0.0,   0.0},
/* Luni-solar terms 311-320 */
{{ 3,  0,  0,  2,  0},        4.9,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{-1,  0,  4,  2,  2},        3.2,     0.0,    0.0,     -1.3,   0.0,   0.0},
{{ 1,  1,  2,  2,  1},        2.3,     0.0,    0.0,     -1.2,   0.0,   0.0},
{{-2,  0,  2,  6,  2},       -4.3,     0.0,    0.0,      1.8,   0.0,   0.0},
{{ 2,  1,  2,  2,  2},        2.6,     0.0,    0.0,     -1.1,   0.0,   0.0},
{{-1,  0,  2,  6,  2},       -3.2,     0.0,    0.0,      1.4,   0.0,   0.0},
{{ 1,  0,  2,  4,  1},       -2.9,     0.0,    0.0,      1.4,   0.0,   0.0},
{{ 2,  0,  2,  4,  2},       -2.7,     0.0,    0.0,      1.2,   0.0,   0.0},
{{ 1,  1, -2,  1,  0},        3.0,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-3,  1,  2,  1,  2},       -1.1,     0.0,    0.0,      0.5,   0.0,   0.0},
/* Luni-solar terms 321-330 */
{{ 2,  0, -2,  0,  2},       -2.1,     0.0,    0.0,      1.0,   0.0,   0.0},
{{-1,  0,  0,  1,  2},       -3.4,     0.0,    0.0,      1.5,   0.0,   0.0},
{{-4,  0,  2,  2,  1},       -1.0,     0.0,    0.0,      0.6,   0.0,   0.0},
{{-1, -1,  0,  1,  0},       -3.6,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  0, -2,  2,  2},       -0.9,     0.0,    0.0,      0.4,   0.0,   0.0},
{{ 1,  0,  0, -1,  2},       -1.2,     0.0,    0.0,      0.5,   0.0,   0.0},
{{ 0, -1,  2, -2,  3},       -2.1,     0.0,    0.0,      0.5,   0.0,   0.0},
{{-2,  1,  2,  0,  0},       -2.9,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 0,  0,  2, -2,  4},       -1.5,     0.0,    0.0,      0.3,   0.0,   0.0},
{{-2, -2,  0,  2,  0},       -2.0,     0.0,    0.0,      0.0,   0.0,   0.0},
/* Luni-solar terms 331-340 */
{{-2,  0, -2,  4,  0},        2.8,     0.0,    0.0,      0.0,   0.0,  -0.2},
{{ 0, -2, -2,  2,  0},        1.7,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1,  2,  0, -2,  1},       -2.2,     0.0,    0.0,      1.2,   0.0,   0.0},
{{ 3,  0,  0, -4,  1},       -1.4,     0.0,    0.0,      0.7,   0.0,   0.0},
{{-1,  1,  2, -2,  2},        2.4,     0.0,    0.0,     -1.1,   0.0,   0.0},
{{ 1, -1,  2, -4,  1},        1.1,     0.0,    0.0,     -0.6,   0.0,   0.0},
{{ 1,  1,  0, -2,  2},        1.4,     0.0,    0.0,     -0.6,   0.0,   0.0},
{{-3,  0,  2,  0,  0},        2.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-3,  0,  2,  0,  2},        1.8,     0.0,    0.0,     -0.8,   0.0,   0.0},
{{-2,  0,  0,  1,  0},       -3.8,     0.0,    0.0,      0.0,   0.0,   0.0},
/* Luni-solar terms 341-350 */
{{ 0,  0, -2,  1,  0},       -3.1,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-3,  0,  0,  2,  1},       -1.6,     0.0,    0.0,      0.8,   0.0,   0.0},
{{-1, -1, -2,  2,  0},        2.9,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  1,  2, -4,  1},       -1.8,     0.0,    0.0,      1.0,   0.0,   0.0},
{{ 2,  1,  0, -4,  1},       -1.0,     0.0,    0.0,      0.5,   0.0,   0.0},
{{ 0,  2,  0, -2,  1},       -1.7,     0.0,    0.0,      1.0,   0.0,   0.0},
{{ 1,  0,  0, -3,  1},        0.9,     0.0,    0.0,     -0.4,   0.0,   0.0},
{{-2,  0,  2, -2,  2},        1.6,     0.0,    0.0,     -0.6,   0.0,   0.0},
{{-2, -1,  0,  0,  1},        2.2,     0.0,    0.0,     -1.2,   0.0,   0.0},
{{-4,  0,  0,  2,  0},        2.0,     0.0,    0.0,      0.0,   0.0,   0.0},
/* Luni-solar terms 351-360 */
{{ 1,  1,  0, -4,  1},       -1.3,     0.0,    0.0,      0.6,   0.0,   0.0},
{{-1,  0,  2, -4,  1},       -1.7,     0.0,    0.0,      0.9,   0.0,   0.0},
{{ 0,  0,  4, -4,  1},       -1.4,     0.0,    0.0,      0.8,   0.0,   0.0},
{{ 0,  3,  2, -2,  2},        0.0,     0.0,    0.0,     -0.7,   0.0,   0.0},
{{-3, -1,  0,  4,  0},        1.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-3,  0,  0,  4,  1},        1.9,     0.0,    0.0,     -1.0,   0.0,   0.0},
{{ 1, -1, -2,  2,  0},       -3.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1, -1,  0,  2,  2},       -2.0,     0.0,    0.0,      0.8,   0.0,   0.0},
{{ 1, -2,  0,  0,  1},        0.9,     0.0,    0.0,     -0.5,   0.0,   0.0},
{{ 1, -1,  0,  0,  2},       -1.8,     0.0,    0.0,      0.7,   0.0,   0.0},
/* Luni-solar terms 361-370 */
{{ 0,  0,  0,  1,  2},        1.3,     0.0,    0.0,     -0.6,   0.0,   0.0},
{{-1, -1,  2,  0,  0},        1.7,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1, -2,  2, -2,  2},       -1.2,     0.0,    0.0,      0.5,   0.0,   0.0},
{{ 0, -1,  2, -1,  1},        1.5,     0.0,    0.0,     -0.8,   0.0,   0.0},
{{-1,  0,  2,  0,  3},       -1.1,     0.0,    0.0,      0.3,   0.0,   0.0},
{{ 1,  1,  0,  0,  2},        1.3,     0.0,    0.0,     -0.5,   0.0,   0.0},
{{-1,  1,  2,  0,  0},       -1.8,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1,  2,  0,  0,  0},       -3.5,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1,  2,  2,  0,  2},        0.9,     0.0,    0.0,     -0.4,   0.0,   0.0},
{{-1,  0,  4, -2,  1},       -1.9,     0.0,    0.0,      1.0,   0.0,   0.0},
/* Luni-solar terms 371-380 */
{{ 3,  0,  2, -4,  2},       -2.6,     0.0,    0.0,      1.1,   0.0,   0.0},
{{ 1,  2,  2, -2,  1},        0.8,     0.0,    0.0,     -0.4,   0.0,   0.0},
{{ 1,  0,  4, -4,  2},       -1.0,     0.0,    0.0,      0.4,   0.0,   0.0},
{{-2, -1,  0,  4,  1},        1.0,     0.0,    0.0,     -0.6,   0.0,   0.0},
{{ 0, -1,  0,  2,  2},       -2.1,     0.0,    0.0,      0.9,   0.0,   0.0},
{{-2,  1,  0,  4,  0},       -1.5,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-2, -1,  2,  2,  1},        0.9,     0.0,    0.0,     -0.5,   0.0,   0.0},
{{ 2,  0, -2,  2,  0},       -2.9,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1,  0,  0,  1,  1},       -1.9,     0.0,    0.0,      1.0,   0.0,   0.0},
{{ 0,  1,  0,  2,  2},        1.2,     0.0,    0.0,     -0.5,   0.0,   0.0},
/* Luni-solar terms 381-390 */
{{ 1, -1,  2, -1,  2},        2.2,     0.0,    0.0,     -0.9,   0.0,   0.0},
{{-2,  0,  4,  0,  1},       -1.0,     0.0,    0.0,      0.5,   0.0,   0.0},
{{ 2,  1,  0,  0,  1},       -2.0,     0.0,    0.0,      1.1,   0.0,   0.0},
{{ 0,  1,  2,  0,  0},       -2.0,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0, -1,  4, -2,  2},       -1.7,     0.0,    0.0,      0.7,   0.0,   0.0},
{{ 0,  0,  4, -2,  4},        1.5,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 0,  2,  2,  0,  1},        0.8,     0.0,    0.0,     -0.4,   0.0,   0.0},
{{-3,  0,  0,  6,  0},        1.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1, -1,  0,  4,  1},       -1.2,     0.0,    0.0,      0.6,   0.0,   0.0},
{{ 1, -2,  0,  2,  0},        2.5,     0.0,    0.0,      0.0,   0.0,   0.0},
/* Luni-solar terms 391-400 */
{{-1,  0,  0,  4,  2},       -1.3,     0.0,    0.0,      0.6,   0.0,   0.0},
{{-1, -2,  2,  2,  1},       -1.4,     0.0,    0.0,      0.8,   0.0,   0.0},
{{-1,  0,  0, -2,  2},        1.3,     0.0,    0.0,     -0.5,   0.0,   0.0},
{{ 1,  0, -2, -2,  1},       -1.7,     0.0,    0.0,      0.9,   0.0,   0.0},
{{ 0,  0, -2, -2,  1},       -1.2,     0.0,    0.0,      0.6,   0.0,   0.0},
{{-2,  0, -2,  0,  1},       -1.0,     0.0,    0.0,      0.5,   0.0,   0.0},
{{ 0,  0,  0,  3,  1},        1.0,     0.0,    0.0,     -0.6,   0.0,   0.0},
{{ 0,  0,  0,  3,  0},       -1.5,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1,  1,  0,  4,  0},       -2.2,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1, -1,  2,  2,  0},        2.8,     0.0,    0.0,     -0.1,   0.0,   0.0},
/* Luni-solar terms 401-410 */
{{-2,  0,  2,  3,  2},        1.5,     0.0,    0.0,     -0.7,   0.0,   0.0},
{{ 1,  0,  0,  2,  2},        2.3,     0.0,    0.0,     -1.0,   0.0,   0.0},
{{ 0, -1,  2,  1,  2},        1.2,     0.0,    0.0,     -0.5,   0.0,   0.0},
{{ 3, -1,  0,  0,  0},        2.9,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 2,  0,  0,  1,  0},       -2.5,     0.0,    0.0,      0.1,   0.0,   0.0},
{{ 1, -1,  2,  0,  0},        2.2,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  0,  2,  1,  0},       -1.8,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1,  0,  2,  0,  3},        1.5,     0.0,    0.0,      0.3,   0.0,   0.0},
{{ 3,  1,  0,  0,  0},       -2.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 3, -1,  2, -2,  2},        1.2,     0.0,    0.0,     -0.5,   0.0,   0.0},
/* Luni-solar terms 411-420 */
{{ 2,  0,  2, -1,  1},       -0.8,     0.0,    0.0,      0.4,   0.0,   0.0},
{{ 1,  1,  2,  0,  0},       -1.9,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  0,  4, -1,  2},       -1.0,     0.0,    0.0,      0.4,   0.0,   0.0},
{{ 1,  2,  2,  0,  2},        2.1,     0.0,    0.0,     -0.9,   0.0,   0.0},
{{-2,  0,  0,  6,  0},        2.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 0, -1,  0,  4,  1},       -1.6,     0.0,    0.0,      0.8,   0.0,   0.0},
{{-2, -1,  2,  4,  1},       -1.9,     0.0,    0.0,      0.9,   0.0,   0.0},
{{ 0, -2,  2,  2,  1},       -2.2,     0.0,    0.0,      1.0,   0.0,   0.0},
{{ 0, -1,  2,  2,  0},        2.7,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{-1,  0,  2,  3,  1},        1.6,     0.0,    0.0,     -0.8,   0.0,   0.0},
/* Luni-solar terms 421-430 */
{{-2,  1,  2,  4,  2},        1.9,     0.0,    0.0,     -0.8,   0.0,   0.0},
{{ 2,  0,  0,  2,  2},        0.9,     0.0,    0.0,     -0.4,   0.0,   0.0},
{{ 2, -2,  2,  0,  2},       -0.9,     0.0,    0.0,      0.4,   0.0,   0.0},
{{-1,  1,  2,  3,  2},       -0.9,     0.0,    0.0,      0.4,   0.0,   0.0},
{{ 3,  0,  2, -1,  2},       -0.8,     0.0,    0.0,      0.4,   0.0,   0.0},
{{ 4,  0,  2, -2,  1},        1.8,     0.0,    0.0,     -0.9,   0.0,   0.0},
{{-1,  0,  0,  6,  0},        1.6,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{-1, -2,  2,  4,  2},       -1.0,     0.0,    0.0,      0.4,   0.0,   0.0},
{{-3,  0,  2,  6,  2},       -2.3,     0.0,    0.0,      0.9,   0.0,   0.0},
{{-1,  0,  2,  4,  0},        1.6,     0.0,    0.0,     -0.1,   0.0,   0.0},
/* Luni-solar terms 431-440 */
{{ 3,  0,  0,  2,  1},       -1.2,     0.0,    0.0,      0.6,   0.0,   0.0},
{{ 3, -1,  2,  0,  1},       -0.8,     0.0,    0.0,      0.4,   0.0,   0.0},
{{ 3,  0,  2,  0,  0},        3.0,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 1,  0,  4,  0,  2},        2.4,     0.0,    0.0,     -1.0,   0.0,   0.0},
{{ 5,  0,  2, -2,  2},        1.0,     0.0,    0.0,     -0.4,   0.0,   0.0},
{{ 0, -1,  2,  4,  1},       -1.6,     0.0,    0.0,      0.7,   0.0,   0.0},
{{ 2, -1,  2,  2,  1},       -1.6,     0.0,    0.0,      0.7,   0.0,   0.0},
{{ 0,  1,  2,  4,  2},        1.7,     0.0,    0.0,     -0.7,   0.0,   0.0},
{{ 1, -1,  2,  4,  2},       -2.4,     0.0,    0.0,      1.0,   0.0,   0.0},
{{ 3, -1,  2,  2,  2},       -1.2,     0.0,    0.0,      0.5,   0.0,   0.0},
/* Luni-solar terms 441-450 */
{{ 3,  0,  2,  2,  1},       -2.4,     0.0,    0.0,      1.1,   0.0,   0.0},
{{ 5,  0,  2,  0,  2},       -2.3,     0.0,    0.0,      0.9,   0.0,   0.0},
{{ 0,  0,  2,  6,  2},       -1.3,     0.0,    0.0,      0.5,   0.0,   0.0},
{{ 4,  0,  2,  2,  2},       -1.5,     0.0,    0.0,      0.7,   0.0,   0.0},
{{ 0, -1,  1, -1,  1},        0.0,     0.0, -198.8,      0.0,   0.0,-167.9},
{{-1,  0,  1,  0,  3},        0.0,     0.0,   -6.3,      0.0,   0.0,  -2.7},
{{ 0, -2,  2, -2,  3},       -0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1,  0, -1,  0,  1},        0.0,     0.0,    0.5,      0.0,   0.0,   0.4},
{{ 2, -2,  0, -2,  1},        0.5,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{-1,  0,  1,  0,  2},        0.0,     0.0,   36.4,      0.0,   0.0,  17.6},
/* Luni-solar terms 451-460 */
{{-1,  0,  1,  0,  1},        0.0,     0.0, -104.4,      0.0,   0.0, -89.1},
{{-1, -1,  2, -1,  2},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{-2,  2,  0,  2,  2},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{-1,  0,  1,  0,  0},        0.0,     0.0,   33.0,      0.0,   0.0,   0.0},
{{-4,  1,  2,  2,  2},        0.5,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{-3,  0,  2,  1,  1},        0.3,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{-2, -1,  2,  0,  2},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{ 1,  0, -2,  1,  1},       -0.5,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 2, -1, -2,  0,  1},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{-4,  0,  2,  2,  0},        0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
/* Luni-solar terms 461-470 */
{{-3,  1,  0,  3,  0},        0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1,  0, -1,  2,  0},        0.0,     0.0,    0.5,      0.0,   0.0,   0.0},
{{ 0, -2,  0,  0,  2},        0.0,     0.0,    0.0,      0.1,   0.0,   0.0},
{{ 0, -2,  0,  0,  2},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{-3,  0,  0,  3,  0},        0.6,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-2, -1,  0,  2,  2},        0.5,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{-1,  0, -2,  3,  0},       -0.7,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-4,  0,  0,  4,  0},       -1.2,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 2,  1, -2,  0,  1},        0.5,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 2, -1,  0, -2,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
/* Luni-solar terms 471-480 */
{{ 0,  0,  1, -1,  0},       -0.5,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1,  2,  0,  1,  0},        0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-2,  1,  2,  0,  2},       -0.7,     0.0,    0.0,      0.3,   0.0,   0.0},
{{ 1,  1,  0, -1,  1},        0.7,     0.0,    0.0,     -0.4,   0.0,   0.0},
{{ 1,  0,  1, -2,  1},        0.0,     0.0,   -1.2,      0.0,   0.0,  -1.0},
{{ 0,  2,  0,  0,  2},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 1, -1,  2, -3,  1},        0.3,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{-1,  1,  2, -1,  1},       -0.3,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-2,  0,  4, -2,  2},       -0.7,     0.0,    0.0,      0.3,   0.0,   0.0},
{{-2,  0,  4, -2,  1},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
/* Luni-solar terms 481-490 */
{{-2, -2,  0,  2,  1},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{-2,  0, -2,  4,  0},        0.0,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1,  2,  2, -4,  1},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{ 1,  1,  2, -4,  2},        0.7,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{-1,  2,  2, -2,  1},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 2,  0,  0, -3,  1},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{-1,  2,  0,  0,  1},       -0.5,     0.0,    0.0,      0.3,   0.0,   0.0},
{{ 0,  0,  0, -2,  0},        0.5,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1, -1,  2, -2,  2},       -0.5,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-1,  1,  0,  0,  2},        0.5,     0.0,    0.0,     -0.2,   0.0,   0.0},
/* Luni-solar terms 491-500 */
{{ 0,  0,  0, -1,  2},       -0.8,     0.0,    0.0,      0.3,   0.0,   0.0},
{{-2,  1,  0,  1,  0},        0.9,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1, -2,  0, -2,  1},        0.6,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 1,  0, -2,  0,  2},       -0.5,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-3,  1,  0,  2,  0},        0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1,  1, -2,  2,  0},       -0.7,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1, -1,  0,  0,  2},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{-3,  0,  0,  2,  0},        0.5,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-3, -1,  0,  2,  0},        0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 2,  0,  2, -6,  1},       -0.3,     0.0,    0.0,      0.2,   0.0,   0.0},
/* Luni-solar terms 501-510 */
{{ 0,  1,  2, -4,  2},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 2,  0,  0, -4,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{-2,  1,  2, -2,  1},       -0.5,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 0, -1,  2, -4,  1},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 0,  1,  0, -2,  2},        0.9,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{-1,  0,  0, -2,  0},        0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 2,  0, -2, -2,  1},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{-4,  0,  2,  0,  1},       -0.3,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-1, -1,  0, -1,  1},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 0,  0, -2,  0,  2},        0.9,     0.0,    0.0,     -0.3,   0.0,   0.0},
/* Luni-solar terms 511-520 */
{{-3,  0,  0,  1,  0},       -0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1,  0, -2,  1,  0},       -0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-2,  0, -2,  2,  1},        0.3,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 0,  0, -4,  2,  0},        0.8,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-2, -1, -2,  2,  0},        0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1,  0,  2, -6,  1},       -0.3,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-1,  0,  2, -4,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 1,  0,  0, -4,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 2,  1,  2, -4,  2},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{ 2,  1,  2, -4,  1},        0.6,     0.0,    0.0,     -0.3,   0.0,   0.0},
/* Luni-solar terms 521-530 */
{{ 0,  1,  4, -4,  4},        0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  1,  4, -4,  2},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{-1, -1, -2,  4,  0},       -0.7,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1, -3,  0,  2,  0},        0.9,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1,  0, -2,  4,  1},       -0.3,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-2, -1,  0,  3,  0},       -0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  0, -2,  3,  0},       -0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-2,  0,  0,  3,  1},       -0.5,     0.0,    0.0,      0.3,   0.0,   0.0},
{{ 0, -1,  0,  1,  0},       -1.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-3,  0,  2,  2,  0},       -0.7,     0.0,    0.0,      0.0,   0.0,   0.0},
/* Luni-solar terms 531-540 */
{{ 1,  1, -2,  2,  0},        1.0,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1,  1,  0,  2,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 1, -2,  2, -2,  1},        1.0,     0.0,    1.3,      0.6,   0.0,  -0.5},
{{ 0,  0,  1,  0,  2},        0.0,     0.0,    3.0,      0.0,   0.0,   1.4},
{{ 0,  0,  1,  0,  1},        0.0,     0.0,  -16.2,      0.0,   0.0, -13.8},
{{ 0,  0,  1,  0,  0},        0.0,     0.0,    7.5,      0.0,   0.0,   0.0},
{{-1,  2,  0,  2,  1},       -0.7,     0.0,    0.0,      0.4,   0.0,   0.0},
{{ 0,  0,  2,  0,  2},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-2,  0,  2,  0,  2},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 2,  0,  0, -1,  1},        0.5,     0.0,    0.0,     -0.2,   0.0,   0.0},
/* Luni-solar terms 541-550 */
{{ 3,  0,  0, -2,  1},        0.5,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 1,  0,  2, -2,  3},       -0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1,  2,  0,  0,  1},       -0.3,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 2,  0,  2, -3,  2},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-1,  1,  4, -2,  2},       -0.5,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-2, -2,  0,  4,  0},        0.6,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0, -3,  0,  2,  0},        0.9,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  0, -2,  4,  0},        0.5,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1, -1,  0,  3,  0},       -0.7,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-2,  0,  0,  4,  2},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
/* Luni-solar terms 551-560 */
{{-1,  0,  0,  3,  1},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 2, -2,  0,  0,  0},        0.7,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1, -1,  0,  1,  0},       -0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1,  0,  0,  2,  0},        0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0, -2,  2,  0,  1},       -0.6,     0.0,   -0.3,      0.3,   0.0,   0.1},
{{-1,  0,  1,  2,  1},        0.0,     0.0,   -0.3,      0.0,   0.0,  -0.2},
{{-1,  1,  0,  3,  0},        1.1,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1, -1,  2,  1,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 0, -1,  2,  0,  0},        1.1,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-2,  1,  2,  2,  1},       -0.3,     0.0,    0.0,      0.2,   0.0,   0.0},
/* Luni-solar terms 561-570 */
{{ 2, -2,  2, -2,  2},       -0.1,     0.0,    0.3,      0.3,   0.0,  -0.1},
{{ 1,  1,  0,  1,  1},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 1,  0,  1,  0,  1},        0.0,     0.0,   -1.3,      0.0,   0.0,  -1.1},
{{ 1,  0,  1,  0,  0},        0.3,     0.0,    0.6,      0.0,   0.0,   0.0},
{{ 0,  2,  0,  2,  0},       -0.7,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 2, -1,  2, -2,  1},        0.5,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 0, -1,  4, -2,  1},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{ 0,  0,  4, -2,  3},        0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  1,  4, -2,  1},        0.5,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 4,  0,  2, -4,  2},       -0.7,     0.0,    0.0,      0.3,   0.0,   0.0},
/* Luni-solar terms 571-580 */
{{ 2,  2,  2, -2,  2},        0.8,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 2,  0,  4, -4,  2},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-1, -2,  0,  4,  0},        1.1,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1, -3,  2,  2,  2},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{-3,  0,  2,  4,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{-3,  0,  2, -2,  1},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-1, -1,  0, -2,  1},        0.8,     0.0,    0.0,     -0.4,   0.0,   0.0},
{{-3,  0,  0,  0,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{-3,  0, -2,  2,  0},        1.1,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  1,  0, -4,  1},       -0.6,     0.0,    0.0,      0.3,   0.0,   0.0},
/* Luni-solar terms 581-590 */
{{-2,  1,  0, -2,  1},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-4,  0,  0,  0,  1},       -0.8,     0.0,    0.0,      0.4,   0.0,   0.0},
{{-1,  0,  0, -4,  1},       -0.7,     0.0,    0.0,      0.3,   0.0,   0.0},
{{-3,  0,  0, -2,  1},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 0,  0,  0,  3,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{-1,  1,  0,  4,  1},        0.6,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 1, -2,  2,  0,  1},       -0.6,     0.0,    0.0,      0.3,   0.0,   0.0},
{{ 0,  1,  0,  3,  0},        0.6,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-1,  0,  2,  2,  3},        0.6,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 0,  0,  2,  2,  2},        0.5,     0.0,    0.0,     -0.2,   0.0,   0.0},
/* Luni-solar terms 591-600 */
{{-2,  0,  2,  2,  2},       -0.5,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-1,  1,  2,  2,  0},       -0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 3,  0,  0,  0,  2},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 2,  1,  0,  1,  0},        0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 2, -1,  2, -1,  2},        0.6,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 0,  0,  2,  0,  1},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 0,  0,  3,  0,  3},        0.0,     0.0,   -2.6,      0.0,   0.0,  -1.1},
{{ 0,  0,  3,  0,  2},        0.0,     0.0,   -1.0,      0.0,   0.0,  -0.5},
{{-1,  2,  2,  2,  1},        0.5,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{-1,  0,  4,  0,  0},       -1.3,     0.0,    0.0,      0.0,   0.0,   0.0},
/* Luni-solar terms 601-610 */
{{ 1,  2,  2,  0,  1},        0.3,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 3,  1,  2, -2,  1},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 1,  1,  4, -2,  2},        0.7,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{-2, -1,  0,  6,  0},        0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0, -2,  0,  4,  0},        0.5,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-2,  0,  0,  6,  1},       -0.3,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-2, -2,  2,  4,  2},       -0.6,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 0, -3,  2,  2,  2},       -0.5,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 0,  0,  0,  4,  2},       -0.7,     0.0,    0.0,      0.3,   0.0,   0.0},
{{-1, -1,  2,  3,  2},        0.5,     0.0,    0.0,     -0.2,   0.0,   0.0},
/* Luni-solar terms 611-620 */
{{-2,  0,  2,  4,  0},        1.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 2, -1,  0,  2,  1},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 1,  0,  0,  3,  0},       -0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  1,  0,  4,  1},        0.5,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 0,  1,  0,  4,  0},       -1.1,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1, -1,  2,  1,  2},        0.5,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 0,  0,  2,  2,  3},        0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1,  0,  2,  2,  2},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{-1,  0,  2,  2,  2},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-2,  0,  4,  2,  1},        0.6,     0.0,    0.0,     -0.3,   0.0,   0.0},
/* Luni-solar terms 621-630 */
{{ 2,  1,  0,  2,  1},        0.3,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 2,  1,  0,  2,  0},       -1.2,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 2, -1,  2,  0,  0},        0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1,  0,  2,  1,  0},       -0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  1,  2,  2,  0},       -0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 2,  0,  2,  0,  3},        0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 3,  0,  2,  0,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 1,  0,  2,  0,  2},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{ 1,  0,  3,  0,  3},        0.0,     0.0,   -0.5,      0.0,   0.0,  -0.2},
{{ 1,  1,  2,  1,  1},       -0.7,     0.0,    0.0,      0.4,   0.0,   0.0},
/* Luni-solar terms 631-640 */
{{ 0,  2,  2,  2,  2},        0.6,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 2,  1,  2,  0,  0},       -0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 2,  0,  4, -2,  1},        0.5,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 4,  1,  2, -2,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{-1, -1,  0,  6,  0},        0.3,     0.0,    0.0,      0.0,   0.0,   0.0},
{{-3, -1,  2,  6,  2},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{-1,  0,  0,  6,  1},       -0.5,     0.0,    0.0,      0.3,   0.0,   0.0},
{{-3,  0,  2,  6,  1},       -0.3,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 1, -1,  0,  4,  1},       -0.3,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 1, -1,  0,  4,  0},        1.2,     0.0,    0.0,      0.0,   0.0,   0.0},
/* Luni-solar terms 641-650 */
{{-2,  0,  2,  5,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 1, -2,  2,  2,  1},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 3, -1,  0,  2,  0},        0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1, -1,  2,  2,  0},        0.6,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  0,  2,  3,  1},        0.5,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{-1,  1,  2,  4,  1},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 0,  1,  2,  3,  2},       -0.6,     0.0,    0.0,      0.3,   0.0,   0.0},
{{-1,  0,  4,  2,  1},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 2,  0,  2,  1,  1},        0.6,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 5,  0,  0,  0,  0},        0.6,     0.0,    0.0,      0.0,   0.0,   0.0},
/* Luni-solar terms 651-660 */
{{ 2,  1,  2,  1,  2},       -0.6,     0.0,    0.0,      0.3,   0.0,   0.0},
{{ 1,  0,  4,  0,  1},        0.3,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 3,  1,  2,  0,  1},        0.7,     0.0,    0.0,     -0.4,   0.0,   0.0},
{{ 3,  0,  4, -2,  2},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{-2, -1,  2,  6,  2},       -0.5,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 0,  0,  0,  6,  0},        0.5,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0, -2,  2,  4,  2},       -0.6,     0.0,    0.0,      0.3,   0.0,   0.0},
{{-2,  0,  2,  6,  1},       -0.6,     0.0,    0.0,      0.3,   0.0,   0.0},
{{ 2,  0,  0,  4,  1},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 2,  0,  0,  4,  0},        1.0,     0.0,    0.0,      0.0,   0.0,   0.0},
/* Luni-solar terms 661-670 */
{{ 2, -2,  2,  2,  2},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 0,  0,  2,  4,  0},        0.7,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 1,  0,  2,  3,  2},        0.7,     0.0,    0.0,     -0.3,   0.0,   0.0},
{{ 4,  0,  0,  2,  0},        0.4,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 2,  0,  2,  2,  0},        1.1,     0.0,    0.0,      0.0,   0.0,   0.0},
{{ 0,  0,  4,  2,  2},        0.5,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 4, -1,  2,  0,  2},       -0.6,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 3,  0,  2,  1,  2},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 2,  1,  2,  2,  1},        0.3,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 4,  1,  2,  0,  2},        0.5,     0.0,    0.0,     -0.2,   0.0,   0.0},
/* Luni-solar terms 671-687 */
{{-1, -1,  2,  6,  2},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{-1,  0,  2,  6,  1},       -0.4,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 1, -1,  2,  4,  1},       -0.3,     0.0,    0.0,      0.2,   0.0,   0.0},
{{ 1,  1,  2,  4,  2},        0.4,     0.0,    0.0,     -0.2,   0.0,   0.0},
{{ 3,  1,  2,  2,  2},        0.3,     0.0,    0.0,     -0.1,   0.0,   0.0},
{{ 5,  0,  2,  0,  1},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{ 2, -1,  2,  4,  2},       -0.3,     0.0,    0.0,      0.1,   0.0,   0.0},
{{ 2,  0,  2,  4,  1},       -0.3,     0.0,    0.0,      0.2,   0.0,   0.0},
   };

#define NLS ( sizeof serls / sizeof serls[0] )

/* ----------------------------- */
/* The planetary nutation series */
/* ----------------------------- */

   static struct {
      int nfa[14];   /* Amount of each fundamental argument */
      int ips;       /* Amplitude of sine term in longitude (0.1uas) */
      int ipc;       /* Amplitude of cosine term in longitude (0.1uas) */
      int ies;       /* Amplitude of sine term in obliquity (0.1uas) */
      int iec;       /* Amplitude of cosine term in obliquity (0.1uas) */
   } serpl[] = {
/* Planetary terms 1-10 */
{{ 0, 0, 0, 0, 0, 0,  0,  8,-16,  4,  5,  0,  0,  0}, 1440,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -8, 16, -4, -5,  0,  0,  2},   56, -117,  -42,  -40},
{{ 0, 0, 0, 0, 0, 0,  0,  8,-16,  4,  5,  0,  0,  2},  125,  -43,    0,  -54},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  0, -1,  2,  2},    0,    5,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -4,  8, -1, -5,  0,  0,  2},    3,   -7,   -3,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  4, -8,  3,  0,  0,  0,  1},    3,    0,    0,   -2},
{{ 0, 0, 1,-1, 1, 0,  0,  3, -8,  3,  0,  0,  0,  0}, -114,    0,    0,   61},
{{-1, 0, 0, 0, 0, 0, 10, -3,  0,  0,  0,  0,  0,  0}, -219,   89,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0, -2,  6, -3,  0,  2},   -3,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  4, -8,  3,  0,  0,  0,  0}, -462, 1604,    0,    0},
/* Planetary terms 11-20 */
{{ 0, 0, 1,-1, 1, 0,  0, -5,  8, -3,  0,  0,  0,  0},   99,    0,    0,  -53},
{{ 0, 0, 0, 0, 0, 0,  0, -4,  8, -3,  0,  0,  0,  1},   -3,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  4, -8,  1,  5,  0,  0,  2},    0,    6,    2,    0},
{{ 0, 0, 0, 0, 0, 0, -5,  6,  4,  0,  0,  0,  0,  2},    3,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  2, -5,  0,  0,  2},  -12,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  2, -5,  0,  0,  1},   14, -218,  117,    8},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  2, -5,  0,  0,  0},   31, -481, -257,  -17},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  2, -5,  0,  0,  0}, -491,  128,    0,    0},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0, -2,  5,  0,  0,  0},-3084, 5123, 2735, 1647},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0, -2,  5,  0,  0,  1},-1444, 2409,-1286, -771},
/* Planetary terms 21-30 */
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0, -2,  5,  0,  0,  2},   11,  -24,  -11,   -9},
{{ 2, 0,-1,-1, 0, 0,  0,  3, -7,  0,  0,  0,  0,  0},   26,   -9,    0,    0},
{{ 1, 0, 0,-2, 0, 0, 19,-21,  3,  0,  0,  0,  0,  0},  103,  -60,    0,    0},
{{ 0, 0, 1,-1, 1, 0,  2, -4,  0, -3,  0,  0,  0,  0},    0,  -13,   -7,    0},
{{ 1, 0, 0,-1, 1, 0,  0, -1,  0,  2,  0,  0,  0,  0},  -26,  -29,  -16,   14},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0, -4, 10,  0,  0,  0},    9,  -27,  -14,   -5},
{{-2, 0, 0, 2, 1, 0,  0,  2,  0,  0, -5,  0,  0,  0},   12,    0,    0,   -6},
{{ 0, 0, 0, 0, 0, 0,  3, -7,  4,  0,  0,  0,  0,  0},   -7,    0,    0,    0},
{{ 0, 0,-1, 1, 0, 0,  0,  1,  0,  1, -1,  0,  0,  0},    0,   24,    0,    0},
{{-2, 0, 0, 2, 1, 0,  0,  2,  0, -2,  0,  0,  0,  0},  284,    0,    0, -151},
/* Planetary terms 31-40 */
{{-1, 0, 0, 0, 0, 0, 18,-16,  0,  0,  0,  0,  0,  0},  226,  101,    0,    0},
{{-2, 0, 1, 1, 2, 0,  0,  1,  0, -2,  0,  0,  0,  0},    0,   -8,   -2,    0},
{{-1, 0, 1,-1, 1, 0, 18,-17,  0,  0,  0,  0,  0,  0},    0,   -6,   -3,    0},
{{-1, 0, 0, 1, 1, 0,  0,  2, -2,  0,  0,  0,  0,  0},    5,    0,    0,   -3},
{{ 0, 0, 0, 0, 0, 0, -8, 13,  0,  0,  0,  0,  0,  2},  -41,  175,   76,   17},
{{ 0, 0, 2,-2, 2, 0, -8, 11,  0,  0,  0,  0,  0,  0},    0,   15,    6,    0},
{{ 0, 0, 0, 0, 0, 0, -8, 13,  0,  0,  0,  0,  0,  1},  425,  212, -133,  269},
{{ 0, 0, 1,-1, 1, 0, -8, 12,  0,  0,  0,  0,  0,  0}, 1200,  598,  319, -641},
{{ 0, 0, 0, 0, 0, 0,  8,-13,  0,  0,  0,  0,  0,  0},  235,  334,    0,    0},
{{ 0, 0, 1,-1, 1, 0,  8,-14,  0,  0,  0,  0,  0,  0},   11,  -12,   -7,   -6},
/* Planetary terms 41-50 */
{{ 0, 0, 0, 0, 0, 0,  8,-13,  0,  0,  0,  0,  0,  1},    5,   -6,    3,    3},
{{-2, 0, 0, 2, 1, 0,  0,  2,  0, -4,  5,  0,  0,  0},   -5,    0,    0,    3},
{{-2, 0, 0, 2, 2, 0,  3, -3,  0,  0,  0,  0,  0,  0},    6,    0,    0,   -3},
{{-2, 0, 0, 2, 0, 0,  0,  2,  0, -3,  1,  0,  0,  0},   15,    0,    0,    0},
{{ 0, 0, 0, 0, 1, 0,  3, -5,  0,  2,  0,  0,  0,  0},   13,    0,    0,   -7},
{{-2, 0, 0, 2, 0, 0,  0,  2,  0, -4,  3,  0,  0,  0},   -6,   -9,    0,    0},
{{ 0, 0,-1, 1, 0, 0,  0,  0,  2,  0,  0,  0,  0,  0},  266,  -78,    0,    0},
{{ 0, 0, 0, 0, 1, 0,  0, -1,  2,  0,  0,  0,  0,  0}, -460, -435, -232,  246},
{{ 0, 0, 1,-1, 2, 0,  0, -2,  2,  0,  0,  0,  0,  0},    0,   15,    7,    0},
{{-1, 0, 1, 0, 1, 0,  3, -5,  0,  0,  0,  0,  0,  0},   -3,    0,    0,    2},
/* Planetary terms 51-60 */
{{-1, 0, 0, 1, 0, 0,  3, -4,  0,  0,  0,  0,  0,  0},    0,  131,    0,    0},
{{-2, 0, 0, 2, 0, 0,  0,  2,  0, -2, -2,  0,  0,  0},    4,    0,    0,    0},
{{-2, 0, 2, 0, 2, 0,  0, -5,  9,  0,  0,  0,  0,  0},    0,    3,    0,    0},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  0,  0, -1,  0,  0},    0,    4,    2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  0,  1,  0,  0},    0,    3,    0,    0},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  0,  0,  0,  2,  0},  -17,  -19,  -10,    9},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  0,  0,  2,  1},   -9,  -11,    6,   -5},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  0,  0,  2,  2},   -6,    0,    0,    3},
{{-1, 0, 0, 1, 0, 0,  0,  3, -4,  0,  0,  0,  0,  0},  -16,    8,    0,    0},
{{ 0, 0,-1, 1, 0, 0,  0,  1,  0,  0,  2,  0,  0,  0},    0,    3,    0,    0},
/* Planetary terms 61-70 */
{{ 0, 0, 1,-1, 2, 0,  0, -1,  0,  0,  2,  0,  0,  0},   11,   24,   11,   -5},
{{ 0, 0, 0, 0, 1, 0,  0, -9, 17,  0,  0,  0,  0,  0},   -3,   -4,   -2,    1},
{{ 0, 0, 0, 0, 2, 0, -3,  5,  0,  0,  0,  0,  0,  0},    3,    0,    0,   -1},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0, -1,  2,  0,  0,  0},    0,   -8,   -4,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  1, -2,  0,  0,  0},    0,    3,    0,    0},
{{ 1, 0, 0,-2, 0, 0, 17,-16,  0, -2,  0,  0,  0,  0},    0,    5,    0,    0},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  1, -3,  0,  0,  0},    0,    3,    2,    0},
{{-2, 0, 0, 2, 1, 0,  0,  5, -6,  0,  0,  0,  0,  0},   -6,    4,    2,    3},
{{ 0, 0,-2, 2, 0, 0,  0,  9,-13,  0,  0,  0,  0,  0},   -3,   -5,    0,    0},
{{ 0, 0, 1,-1, 2, 0,  0, -1,  0,  0,  1,  0,  0,  0},   -5,    0,    0,    2},
/* Planetary terms 71-80 */
{{ 0, 0, 0, 0, 1, 0,  0,  0,  0,  0,  1,  0,  0,  0},    4,   24,   13,   -2},
{{ 0, 0,-1, 1, 0, 0,  0,  1,  0,  0,  1,  0,  0,  0},  -42,   20,    0,    0},
{{ 0, 0,-2, 2, 0, 0,  5, -6,  0,  0,  0,  0,  0,  0},  -10,  233,    0,    0},
{{ 0, 0,-1, 1, 1, 0,  5, -7,  0,  0,  0,  0,  0,  0},   -3,    0,    0,    1},
{{-2, 0, 0, 2, 0, 0,  6, -8,  0,  0,  0,  0,  0,  0},   78,  -18,    0,    0},
{{ 2, 0, 1,-3, 1, 0, -6,  7,  0,  0,  0,  0,  0,  0},    0,    3,    1,    0},
{{ 0, 0, 0, 0, 2, 0,  0,  0,  0,  1,  0,  0,  0,  0},    0,   -3,   -1,    0},
{{ 0, 0,-1, 1, 1, 0,  0,  1,  0,  1,  0,  0,  0,  0},    0,   -4,   -2,    1},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  0,  0,  2,  0,  0},    0,   -8,   -4,   -1},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  0,  2,  0,  1},    0,   -5,    3,    0},
/* Planetary terms 81-90 */
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  0,  2,  0,  2},   -7,    0,    0,    3},
{{ 0, 0, 0, 0, 0, 0,  0, -8, 15,  0,  0,  0,  0,  2},  -14,    8,    3,    6},
{{ 0, 0, 0, 0, 0, 0,  0, -8, 15,  0,  0,  0,  0,  1},    0,    8,   -4,    0},
{{ 0, 0, 1,-1, 1, 0,  0, -9, 15,  0,  0,  0,  0,  0},    0,   19,   10,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  8,-15,  0,  0,  0,  0,  0},   45,  -22,    0,    0},
{{ 1, 0,-1,-1, 0, 0,  0,  8,-15,  0,  0,  0,  0,  0},   -3,    0,    0,    0},
{{ 2, 0, 0,-2, 0, 0,  2, -5,  0,  0,  0,  0,  0,  0},    0,   -3,    0,    0},
{{-2, 0, 0, 2, 0, 0,  0,  2,  0, -5,  5,  0,  0,  0},    0,    3,    0,    0},
{{ 2, 0, 0,-2, 1, 0,  0, -6,  8,  0,  0,  0,  0,  0},    3,    5,    3,   -2},
{{ 2, 0, 0,-2, 1, 0,  0, -2,  0,  3,  0,  0,  0,  0},   89,  -16,   -9,  -48},
/* Planetary terms 91-100 */
{{-2, 0, 1, 1, 0, 0,  0,  1,  0, -3,  0,  0,  0,  0},    0,    3,    0,    0},
{{-2, 0, 1, 1, 1, 0,  0,  1,  0, -3,  0,  0,  0,  0},   -3,    7,    4,    2},
{{-2, 0, 0, 2, 0, 0,  0,  2,  0, -3,  0,  0,  0,  0}, -349,  -62,    0,    0},
{{-2, 0, 0, 2, 0, 0,  0,  6, -8,  0,  0,  0,  0,  0},  -15,   22,    0,    0},
{{-2, 0, 0, 2, 0, 0,  0,  2,  0, -1, -5,  0,  0,  0},   -3,    0,    0,    0},
{{-1, 0, 0, 1, 0, 0,  0,  1,  0, -1,  0,  0,  0,  0},  -53,    0,    0,    0},
{{-1, 0, 1, 1, 1, 0,-20, 20,  0,  0,  0,  0,  0,  0},    5,    0,    0,   -3},
{{ 1, 0, 0,-2, 0, 0, 20,-21,  0,  0,  0,  0,  0,  0},    0,   -8,    0,    0},
{{ 0, 0, 0, 0, 1, 0,  0,  8,-15,  0,  0,  0,  0,  0},   15,   -7,   -4,   -8},
{{ 0, 0, 2,-2, 1, 0,  0,-10, 15,  0,  0,  0,  0,  0},   -3,    0,    0,    1},
/* Planetary terms 101-110 */
{{ 0, 0,-1, 1, 0, 0,  0,  1,  0,  1,  0,  0,  0,  0},  -21,  -78,    0,    0},
{{ 0, 0, 0, 0, 1, 0,  0,  0,  0,  1,  0,  0,  0,  0},   20,  -70,  -37,  -11},
{{ 0, 0, 1,-1, 2, 0,  0, -1,  0,  1,  0,  0,  0,  0},    0,    6,    3,    0},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0, -2,  4,  0,  0,  0},    5,    3,    2,   -2},
{{ 2, 0, 0,-2, 1, 0, -6,  8,  0,  0,  0,  0,  0,  0},  -17,   -4,   -2,    9},
{{ 0, 0,-2, 2, 1, 0,  5, -6,  0,  0,  0,  0,  0,  0},    0,    6,    3,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0, -1,  0,  0,  1},   32,   15,   -8,   17},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  0, -1,  0,  0,  0},  174,   84,   45,  -93},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  1,  0,  0,  0},   11,   56,    0,    0},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  0,  1,  0,  0,  0},  -66,  -12,   -6,   35},
/* Planetary terms 111-120 */
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  1,  0,  0,  1},   47,    8,    4,  -25},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  1,  0,  0,  2},    0,    8,    4,    0},
{{ 0, 0, 2,-2, 1, 0,  0, -9, 13,  0,  0,  0,  0,  0},   10,  -22,  -12,   -5},
{{ 0, 0, 0, 0, 1, 0,  0,  7,-13,  0,  0,  0,  0,  0},   -3,    0,    0,    2},
{{-2, 0, 0, 2, 0, 0,  0,  5, -6,  0,  0,  0,  0,  0},  -24,   12,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  9,-17,  0,  0,  0,  0,  0},    5,   -6,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -9, 17,  0,  0,  0,  0,  2},    3,    0,    0,   -2},
{{ 1, 0, 0,-1, 1, 0,  0, -3,  4,  0,  0,  0,  0,  0},    4,    3,    1,   -2},
{{ 1, 0, 0,-1, 1, 0, -3,  4,  0,  0,  0,  0,  0,  0},    0,   29,   15,    0},
{{ 0, 0, 0, 0, 2, 0,  0, -1,  2,  0,  0,  0,  0,  0},   -5,   -4,   -2,    2},
/* Planetary terms 121-130 */
{{ 0, 0,-1, 1, 1, 0,  0,  0,  2,  0,  0,  0,  0,  0},    8,   -3,   -1,   -5},
{{ 0, 0,-2, 2, 0, 1,  0, -2,  0,  0,  0,  0,  0,  0},    0,   -3,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  3, -5,  0,  2,  0,  0,  0,  0},   10,    0,    0,    0},
{{-2, 0, 0, 2, 1, 0,  0,  2,  0, -3,  1,  0,  0,  0},    3,    0,    0,   -2},
{{-2, 0, 0, 2, 1, 0,  3, -3,  0,  0,  0,  0,  0,  0},   -5,    0,    0,    3},
{{ 0, 0, 0, 0, 1, 0,  8,-13,  0,  0,  0,  0,  0,  0},   46,   66,   35,  -25},
{{ 0, 0,-1, 1, 0, 0,  8,-12,  0,  0,  0,  0,  0,  0},  -14,    7,    0,    0},
{{ 0, 0, 2,-2, 1, 0, -8, 11,  0,  0,  0,  0,  0,  0},    0,    3,    2,    0},
{{-1, 0, 0, 1, 0, 0,  0,  2, -2,  0,  0,  0,  0,  0},   -5,    0,    0,    0},
{{-1, 0, 0, 0, 1, 0, 18,-16,  0,  0,  0,  0,  0,  0},  -68,  -34,  -18,   36},
/* Planetary terms 131-140 */
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0, -1,  1,  0,  0,  0},    0,   14,    7,    0},
{{ 0, 0, 0, 0, 1, 0,  3, -7,  4,  0,  0,  0,  0,  0},   10,   -6,   -3,   -5},
{{-2, 0, 1, 1, 1, 0,  0, -3,  7,  0,  0,  0,  0,  0},   -5,   -4,   -2,    3},
{{ 0, 0, 1,-1, 2, 0,  0, -1,  0, -2,  5,  0,  0,  0},   -3,    5,    2,    1},
{{ 0, 0, 0, 0, 1, 0,  0,  0,  0, -2,  5,  0,  0,  0},   76,   17,    9,  -41},
{{ 0, 0, 0, 0, 1, 0,  0, -4,  8, -3,  0,  0,  0,  0},   84,  298,  159,  -45},
{{ 1, 0, 0, 0, 1, 0,-10,  3,  0,  0,  0,  0,  0,  0},    3,    0,    0,   -1},
{{ 0, 0, 2,-2, 1, 0,  0, -2,  0,  0,  0,  0,  0,  0},   -3,    0,    0,    2},
{{-1, 0, 0, 0, 1, 0, 10, -3,  0,  0,  0,  0,  0,  0},   -3,    0,    0,    1},
{{ 0, 0, 0, 0, 1, 0,  0,  4, -8,  3,  0,  0,  0,  0},  -82,  292,  156,   44},
/* Planetary terms 141-150 */
{{ 0, 0, 0, 0, 1, 0,  0,  0,  0,  2, -5,  0,  0,  0},  -73,   17,    9,   39},
{{ 0, 0,-1, 1, 0, 0,  0,  1,  0,  2, -5,  0,  0,  0},   -9,  -16,    0,    0},
{{ 2, 0,-1,-1, 1, 0,  0,  3, -7,  0,  0,  0,  0,  0},    3,    0,   -1,   -2},
{{-2, 0, 0, 2, 0, 0,  0,  2,  0,  0, -5,  0,  0,  0},   -3,    0,    0,    0},
{{ 0, 0, 0, 0, 1, 0, -3,  7, -4,  0,  0,  0,  0,  0},   -9,   -5,   -3,    5},
{{-2, 0, 0, 2, 0, 0,  0,  2,  0, -2,  0,  0,  0,  0}, -439,    0,    0,    0},
{{ 1, 0, 0, 0, 1, 0,-18, 16,  0,  0,  0,  0,  0,  0},   57,  -28,  -15,  -30},
{{-2, 0, 1, 1, 1, 0,  0,  1,  0, -2,  0,  0,  0,  0},    0,   -6,   -3,    0},
{{ 0, 0, 1,-1, 2, 0, -8, 12,  0,  0,  0,  0,  0,  0},   -4,    0,    0,    2},
{{ 0, 0, 0, 0, 1, 0, -8, 13,  0,  0,  0,  0,  0,  0},  -40,   57,   30,   21},
/* Planetary terms 151-160 */
{{ 0, 0, 0, 0, 0, 0,  0,  1, -2,  0,  0,  0,  0,  1},   23,    7,    3,  -13},
{{ 0, 0, 1,-1, 1, 0,  0,  0, -2,  0,  0,  0,  0,  0},  273,   80,   43, -146},
{{ 0, 0, 0, 0, 0, 0,  0,  1, -2,  0,  0,  0,  0,  0}, -449,  430,    0,    0},
{{ 0, 0, 1,-1, 1, 0,  0, -2,  2,  0,  0,  0,  0,  0},   -8,  -47,  -25,    4},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  2,  0,  0,  0,  0,  1},    6,   47,   25,   -3},
{{-1, 0, 0, 1, 1, 0,  3, -4,  0,  0,  0,  0,  0,  0},    0,   23,   13,    0},
{{-1, 0, 0, 1, 1, 0,  0,  3, -4,  0,  0,  0,  0,  0},   -3,    0,    0,    2},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  0, -2,  0,  0,  0},    3,   -4,   -2,   -2},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  0,  2,  0,  0,  0},  -48, -110,  -59,   26},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  2,  0,  0,  1},   51,  114,   61,  -27},
/* Planetary terms 161-170 */
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  2,  0,  0,  2}, -133,    0,    0,   57},
{{ 0, 0, 1,-1, 0, 0,  3, -6,  0,  0,  0,  0,  0,  0},    0,    4,    0,    0},
{{ 0, 0, 0, 0, 1, 0, -3,  5,  0,  0,  0,  0,  0,  0},  -21,   -6,   -3,   11},
{{ 0, 0, 1,-1, 2, 0, -3,  4,  0,  0,  0,  0,  0,  0},    0,   -3,   -1,    0},
{{ 0, 0, 0, 0, 1, 0,  0, -2,  4,  0,  0,  0,  0,  0},  -11,  -21,  -11,    6},
{{ 0, 0, 2,-2, 1, 0, -5,  6,  0,  0,  0,  0,  0,  0},  -18, -436, -233,    9},
{{ 0, 0,-1, 1, 0, 0,  5, -7,  0,  0,  0,  0,  0,  0},   35,   -7,    0,    0},
{{ 0, 0, 0, 0, 1, 0,  5, -8,  0,  0,  0,  0,  0,  0},    0,    5,    3,    0},
{{-2, 0, 0, 2, 1, 0,  6, -8,  0,  0,  0,  0,  0,  0},   11,   -3,   -1,   -6},
{{ 0, 0, 0, 0, 1, 0,  0, -8, 15,  0,  0,  0,  0,  0},   -5,   -3,   -1,    3},
/* Planetary terms 171-180 */
{{-2, 0, 0, 2, 1, 0,  0,  2,  0, -3,  0,  0,  0,  0},  -53,   -9,   -5,   28},
{{-2, 0, 0, 2, 1, 0,  0,  6, -8,  0,  0,  0,  0,  0},    0,    3,    2,    1},
{{ 1, 0, 0,-1, 1, 0,  0, -1,  0,  1,  0,  0,  0,  0},    4,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  3, -5,  0,  0,  0},    0,   -4,    0,    0},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0, -1,  0,  0,  0,  0},  -50,  194,  103,   27},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0, -1,  0,  0,  0,  1},  -13,   52,   28,    7},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  1,  0,  0,  0,  0},  -91,  248,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  1,  0,  0,  0,  1},    6,   49,   26,   -3},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  1,  0,  0,  0,  0},   -6,  -47,  -25,    3},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  1,  0,  0,  0,  1},    0,    5,    3,    0},
/* Planetary terms 181-190 */
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  1,  0,  0,  0,  2},   52,   23,   10,  -23},
{{ 0, 0, 1,-1, 2, 0,  0, -1,  0,  0, -1,  0,  0,  0},   -3,    0,    0,    1},
{{ 0, 0, 0, 0, 1, 0,  0,  0,  0,  0, -1,  0,  0,  0},    0,    5,    3,    0},
{{ 0, 0,-1, 1, 0, 0,  0,  1,  0,  0, -1,  0,  0,  0},   -4,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -7, 13,  0,  0,  0,  0,  2},   -4,    8,    3,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  7,-13,  0,  0,  0,  0,  0},   10,    0,    0,    0},
{{ 2, 0, 0,-2, 1, 0,  0, -5,  6,  0,  0,  0,  0,  0},    3,    0,    0,   -2},
{{ 0, 0, 2,-2, 1, 0,  0, -8, 11,  0,  0,  0,  0,  0},    0,    8,    4,    0},
{{ 0, 0, 2,-2, 1,-1,  0,  2,  0,  0,  0,  0,  0,  0},    0,    8,    4,    1},
{{-2, 0, 0, 2, 0, 0,  0,  4, -4,  0,  0,  0,  0,  0},   -4,    0,    0,    0},
/* Planetary terms 191-200 */
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  2, -2,  0,  0,  0},   -4,    0,    0,    0},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  0,  3,  0,  0,  0},   -8,    4,    2,    4},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  3,  0,  0,  1},    8,   -4,   -2,   -4},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  3,  0,  0,  2},    0,   15,    7,    0},
{{-2, 0, 0, 2, 0, 0,  3, -3,  0,  0,  0,  0,  0,  0}, -138,    0,    0,    0},
{{ 0, 0, 0, 0, 2, 0,  0, -4,  8, -3,  0,  0,  0,  0},    0,   -7,   -3,    0},
{{ 0, 0, 0, 0, 2, 0,  0,  4, -8,  3,  0,  0,  0,  0},    0,   -7,   -3,    0},
{{ 2, 0, 0,-2, 1, 0,  0, -2,  0,  2,  0,  0,  0,  0},   54,    0,    0,  -29},
{{ 0, 0, 1,-1, 2, 0,  0, -1,  0,  2,  0,  0,  0,  0},    0,   10,    4,    0},
{{ 0, 0, 1,-1, 2, 0,  0,  0, -2,  0,  0,  0,  0,  0},   -7,    0,    0,    3},
/* Planetary terms 201-210 */
{{ 0, 0, 0, 0, 1, 0,  0,  1, -2,  0,  0,  0,  0,  0},  -37,   35,   19,   20},
{{ 0, 0,-1, 1, 0, 0,  0,  2, -2,  0,  0,  0,  0,  0},    0,    4,    0,    0},
{{ 0, 0,-1, 1, 0, 0,  0,  1,  0,  0, -2,  0,  0,  0},   -4,    9,    0,    0},
{{ 0, 0, 2,-2, 1, 0,  0, -2,  0,  0,  2,  0,  0,  0},    8,    0,    0,   -4},
{{ 0, 0, 1,-1, 1, 0,  3, -6,  0,  0,  0,  0,  0,  0},   -9,  -14,   -8,    5},
{{ 0, 0, 0, 0, 0, 0,  3, -5,  0,  0,  0,  0,  0,  1},   -3,   -9,   -5,    3},
{{ 0, 0, 0, 0, 0, 0,  3, -5,  0,  0,  0,  0,  0,  0}, -145,   47,    0,    0},
{{ 0, 0, 1,-1, 1, 0, -3,  4,  0,  0,  0,  0,  0,  0},  -10,   40,   21,    5},
{{ 0, 0, 0, 0, 0, 0, -3,  5,  0,  0,  0,  0,  0,  1},   11,  -49,  -26,   -7},
{{ 0, 0, 0, 0, 0, 0, -3,  5,  0,  0,  0,  0,  0,  2},-2150,    0,    0,  932},
/* Planetary terms 211-220 */
{{ 0, 0, 2,-2, 2, 0, -3,  3,  0,  0,  0,  0,  0,  0},  -12,    0,    0,    5},
{{ 0, 0, 0, 0, 0, 0, -3,  5,  0,  0,  0,  0,  0,  2},   85,    0,    0,  -37},
{{ 0, 0, 0, 0, 0, 0,  0,  2, -4,  0,  0,  0,  0,  1},    4,    0,    0,   -2},
{{ 0, 0, 1,-1, 1, 0,  0,  1, -4,  0,  0,  0,  0,  0},    3,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  2, -4,  0,  0,  0,  0,  0},  -86,  153,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -2,  4,  0,  0,  0,  0,  1},   -6,    9,    5,    3},
{{ 0, 0, 1,-1, 1, 0,  0, -3,  4,  0,  0,  0,  0,  0},    9,  -13,   -7,   -5},
{{ 0, 0, 0, 0, 0, 0,  0, -2,  4,  0,  0,  0,  0,  1},   -8,   12,    6,    4},
{{ 0, 0, 0, 0, 0, 0,  0, -2,  4,  0,  0,  0,  0,  2},  -51,    0,    0,   22},
{{ 0, 0, 0, 0, 0, 0, -5,  8,  0,  0,  0,  0,  0,  2},  -11, -268, -116,    5},
/* Planetary terms 221-230 */
{{ 0, 0, 2,-2, 2, 0, -5,  6,  0,  0,  0,  0,  0,  0},    0,   12,    5,    0},
{{ 0, 0, 0, 0, 0, 0, -5,  8,  0,  0,  0,  0,  0,  2},    0,    7,    3,    0},
{{ 0, 0, 0, 0, 0, 0, -5,  8,  0,  0,  0,  0,  0,  1},   31,    6,    3,  -17},
{{ 0, 0, 1,-1, 1, 0, -5,  7,  0,  0,  0,  0,  0,  0},  140,   27,   14,  -75},
{{ 0, 0, 0, 0, 0, 0, -5,  8,  0,  0,  0,  0,  0,  1},   57,   11,    6,  -30},
{{ 0, 0, 0, 0, 0, 0,  5, -8,  0,  0,  0,  0,  0,  0},  -14,  -39,    0,    0},
{{ 0, 0, 1,-1, 2, 0,  0, -1,  0, -1,  0,  0,  0,  0},    0,   -6,   -2,    0},
{{ 0, 0, 0, 0, 1, 0,  0,  0,  0, -1,  0,  0,  0,  0},    4,   15,    8,   -2},
{{ 0, 0,-1, 1, 0, 0,  0,  1,  0, -1,  0,  0,  0,  0},    0,    4,    0,    0},
{{ 0, 0, 2,-2, 1, 0,  0, -2,  0,  1,  0,  0,  0,  0},   -3,    0,    0,    1},
/* Planetary terms 231-240 */
{{ 0, 0, 0, 0, 0, 0,  0, -6, 11,  0,  0,  0,  0,  2},    0,   11,    5,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  6,-11,  0,  0,  0,  0,  0},    9,    6,    0,    0},
{{ 0, 0, 0, 0, 0,-1,  0,  4,  0,  0,  0,  0,  0,  2},   -4,   10,    4,    2},
{{ 0, 0, 0, 0, 0, 1,  0, -4,  0,  0,  0,  0,  0,  0},    5,    3,    0,    0},
{{ 2, 0, 0,-2, 1, 0, -3,  3,  0,  0,  0,  0,  0,  0},   16,    0,    0,   -9},
{{-2, 0, 0, 2, 0, 0,  0,  2,  0,  0, -2,  0,  0,  0},   -3,    0,    0,    0},
{{ 0, 0, 2,-2, 1, 0,  0, -7,  9,  0,  0,  0,  0,  0},    0,    3,    2,   -1},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  4, -5,  0,  0,  2},    7,    0,    0,   -3},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  2,  0,  0,  0,  0},  -25,   22,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  2,  0,  0,  0,  1},   42,  223,  119,  -22},
/* Planetary terms 241-250 */
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  2,  0,  0,  0,  0},  -27, -143,  -77,   14},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  2,  0,  0,  0,  1},    9,   49,   26,   -5},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  2,  0,  0,  0,  2},-1166,    0,    0,  505},
{{ 0, 0, 2,-2, 2, 0,  0, -2,  0,  2,  0,  0,  0,  0},   -5,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  5,  0,  0,  2},   -6,    0,    0,    3},
{{ 0, 0, 0, 0, 1, 0,  3, -5,  0,  0,  0,  0,  0,  0},   -8,    0,    1,    4},
{{ 0, 0,-1, 1, 0, 0,  3, -4,  0,  0,  0,  0,  0,  0},    0,   -4,    0,    0},
{{ 0, 0, 2,-2, 1, 0, -3,  3,  0,  0,  0,  0,  0,  0},  117,    0,    0,  -63},
{{ 0, 0, 0, 0, 1, 0,  0,  2, -4,  0,  0,  0,  0,  0},   -4,    8,    4,    2},
{{ 0, 0, 2,-2, 1, 0,  0, -4,  4,  0,  0,  0,  0,  0},    3,    0,    0,   -2},
/* Planetary terms 251-260 */
{{ 0, 0, 1,-1, 2, 0, -5,  7,  0,  0,  0,  0,  0,  0},   -5,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  3, -6,  0,  0,  0,  0,  0},    0,   31,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -3,  6,  0,  0,  0,  0,  1},   -5,    0,    1,    3},
{{ 0, 0, 1,-1, 1, 0,  0, -4,  6,  0,  0,  0,  0,  0},    4,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0,  0, -3,  6,  0,  0,  0,  0,  1},   -4,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0,  0, -3,  6,  0,  0,  0,  0,  2},  -24,  -13,   -6,   10},
{{ 0, 0,-1, 1, 0, 0,  2, -2,  0,  0,  0,  0,  0,  0},    3,    0,    0,    0},
{{ 0, 0, 0, 0, 1, 0,  2, -3,  0,  0,  0,  0,  0,  0},    0,  -32,  -17,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -5,  9,  0,  0,  0,  0,  2},    8,   12,    5,   -3},
{{ 0, 0, 0, 0, 0, 0,  0, -5,  9,  0,  0,  0,  0,  1},    3,    0,    0,   -1},
/* Planetary terms 261-270 */
{{ 0, 0, 0, 0, 0, 0,  0,  5, -9,  0,  0,  0,  0,  0},    7,   13,    0,    0},
{{ 0, 0,-1, 1, 0, 0,  0,  1,  0, -2,  0,  0,  0,  0},   -3,   16,    0,    0},
{{ 0, 0, 2,-2, 1, 0,  0, -2,  0,  2,  0,  0,  0,  0},   50,    0,    0,  -27},
{{-2, 0, 1, 1, 1, 0,  0,  1,  0,  0,  0,  0,  0,  0},    0,   -5,   -3,    0},
{{ 0, 0,-2, 2, 0, 0,  3, -3,  0,  0,  0,  0,  0,  0},   13,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0, -6, 10,  0,  0,  0,  0,  0,  1},    0,    5,    3,    1},
{{ 0, 0, 0, 0, 0, 0, -6, 10,  0,  0,  0,  0,  0,  2},   24,    5,    2,  -11},
{{ 0, 0, 0, 0, 0, 0, -2,  3,  0,  0,  0,  0,  0,  2},    5,  -11,   -5,   -2},
{{ 0, 0, 0, 0, 0, 0, -2,  3,  0,  0,  0,  0,  0,  1},   30,   -3,   -2,  -16},
{{ 0, 0, 1,-1, 1, 0, -2,  2,  0,  0,  0,  0,  0,  0},   18,    0,    0,   -9},
/* Planetary terms 271-280 */
{{ 0, 0, 0, 0, 0, 0,  2, -3,  0,  0,  0,  0,  0,  0},    8,  614,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  2, -3,  0,  0,  0,  0,  0,  1},    3,   -3,   -1,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  3,  0,  0,  0,  1},    6,   17,    9,   -3},
{{ 0, 0, 1,-1, 1, 0,  0, -1,  0,  3,  0,  0,  0,  0},   -3,   -9,   -5,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  3,  0,  0,  0,  1},    0,    6,    3,   -1},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  3,  0,  0,  0,  2}, -127,   21,    9,   55},
{{ 0, 0, 0, 0, 0, 0,  0,  4, -8,  0,  0,  0,  0,  0},    3,    5,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -4,  8,  0,  0,  0,  0,  2},   -6,  -10,   -4,    3},
{{ 0, 0,-2, 2, 0, 0,  0,  2,  0, -2,  0,  0,  0,  0},    5,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -4,  7,  0,  0,  0,  0,  2},   16,    9,    4,   -7},
/* Planetary terms 281-290 */
{{ 0, 0, 0, 0, 0, 0,  0, -4,  7,  0,  0,  0,  0,  1},    3,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  4, -7,  0,  0,  0,  0,  0},    0,   22,    0,    0},
{{ 0, 0, 0, 0, 1, 0, -2,  3,  0,  0,  0,  0,  0,  0},    0,   19,   10,    0},
{{ 0, 0, 2,-2, 1, 0,  0, -2,  0,  3,  0,  0,  0,  0},    7,    0,    0,   -4},
{{ 0, 0, 0, 0, 0, 0,  0, -5, 10,  0,  0,  0,  0,  2},    0,   -5,   -2,    0},
{{ 0, 0, 0, 0, 1, 0, -1,  2,  0,  0,  0,  0,  0,  0},    0,    3,    1,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  4,  0,  0,  0,  2},   -9,    3,    1,    4},
{{ 0, 0, 0, 0, 0, 0,  0, -3,  5,  0,  0,  0,  0,  2},   17,    0,    0,   -7},
{{ 0, 0, 0, 0, 0, 0,  0, -3,  5,  0,  0,  0,  0,  1},    0,   -3,   -2,   -1},
{{ 0, 0, 0, 0, 0, 0,  0,  3, -5,  0,  0,  0,  0,  0},  -20,   34,    0,    0},
/* Planetary terms 291-300 */
{{ 0, 0, 0, 0, 0, 0,  1, -2,  0,  0,  0,  0,  0,  1},  -10,    0,    1,    5},
{{ 0, 0, 1,-1, 1, 0,  1, -3,  0,  0,  0,  0,  0,  0},   -4,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0,  1, -2,  0,  0,  0,  0,  0,  0},   22,  -87,    0,    0},
{{ 0, 0, 0, 0, 0, 0, -1,  2,  0,  0,  0,  0,  0,  1},   -4,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0, -1,  2,  0,  0,  0,  0,  0,  2},   -3,   -6,   -2,    1},
{{ 0, 0, 0, 0, 0, 0, -7, 11,  0,  0,  0,  0,  0,  2},  -16,   -3,   -1,    7},
{{ 0, 0, 0, 0, 0, 0, -7, 11,  0,  0,  0,  0,  0,  1},    0,   -3,   -2,    0},
{{ 0, 0,-2, 2, 0, 0,  4, -4,  0,  0,  0,  0,  0,  0},    4,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  2, -3,  0,  0,  0,  0,  0},  -68,   39,    0,    0},
{{ 0, 0, 2,-2, 1, 0, -4,  4,  0,  0,  0,  0,  0,  0},   27,    0,    0,  -14},
/* Planetary terms 301-310 */
{{ 0, 0,-1, 1, 0, 0,  4, -5,  0,  0,  0,  0,  0,  0},    0,   -4,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1, -1,  0,  0,  0,  0,  0},  -25,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0, -4,  7,  0,  0,  0,  0,  0,  1},  -12,   -3,   -2,    6},
{{ 0, 0, 1,-1, 1, 0, -4,  6,  0,  0,  0,  0,  0,  0},    3,    0,    0,   -1},
{{ 0, 0, 0, 0, 0, 0, -4,  7,  0,  0,  0,  0,  0,  2},    3,   66,   29,   -1},
{{ 0, 0, 0, 0, 0, 0, -4,  6,  0,  0,  0,  0,  0,  2},  490,    0,    0, -213},
{{ 0, 0, 0, 0, 0, 0, -4,  6,  0,  0,  0,  0,  0,  1},  -22,   93,   49,   12},
{{ 0, 0, 1,-1, 1, 0, -4,  5,  0,  0,  0,  0,  0,  0},   -7,   28,   15,    4},
{{ 0, 0, 0, 0, 0, 0, -4,  6,  0,  0,  0,  0,  0,  1},   -3,   13,    7,    2},
{{ 0, 0, 0, 0, 0, 0,  4, -6,  0,  0,  0,  0,  0,  0},  -46,   14,    0,    0},
/* Planetary terms 311-320 */
{{-2, 0, 0, 2, 0, 0,  2, -2,  0,  0,  0,  0,  0,  0},   -5,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  1,  0,  0,  0,  0,  0},    2,    1,    0,    0},
{{ 0, 0,-1, 1, 0, 0,  1,  0,  0,  0,  0,  0,  0,  0},    0,   -3,    0,    0},
{{ 0, 0, 0, 0, 1, 0,  1, -1,  0,  0,  0,  0,  0,  0},  -28,    0,    0,   15},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  0,  5,  0,  0,  0,  2},    5,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  1, -3,  0,  0,  0,  0,  0},    0,    3,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  3,  0,  0,  0,  0,  2},  -11,    0,    0,    5},
{{ 0, 0, 0, 0, 0, 0,  0, -7, 12,  0,  0,  0,  0,  2},    0,    3,    1,    0},
{{ 0, 0, 0, 0, 0, 0, -1,  1,  0,  0,  0,  0,  0,  2},   -3,    0,    0,    1},
{{ 0, 0, 0, 0, 0, 0, -1,  1,  0,  0,  0,  0,  0,  1},   25,  106,   57,  -13},
/* Planetary terms 321-330 */
{{ 0, 0, 1,-1, 1, 0, -1,  0,  0,  0,  0,  0,  0,  0},    5,   21,   11,   -3},
{{ 0, 0, 0, 0, 0, 0,  1, -1,  0,  0,  0,  0,  0,  0}, 1485,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  1, -1,  0,  0,  0,  0,  0,  1},   -7,  -32,  -17,    4},
{{ 0, 0, 1,-1, 1, 0,  1, -2,  0,  0,  0,  0,  0,  0},    0,    5,    3,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -2,  5,  0,  0,  0,  0,  2},   -6,   -3,   -2,    3},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  0,  4,  0,  0,  0,  2},   30,   -6,   -2,  -13},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0, -4,  0,  0,  0,  0},   -4,    4,    0,    0},
{{ 0, 0, 0, 0, 1, 0, -1,  1,  0,  0,  0,  0,  0,  0},  -19,    0,    0,   10},
{{ 0, 0, 0, 0, 0, 0,  0, -6, 10,  0,  0,  0,  0,  2},    0,    4,    2,   -1},
{{ 0, 0, 0, 0, 0, 0,  0, -6, 10,  0,  0,  0,  0,  0},    0,    3,    0,    0},
/* Planetary terms 331-340 */
{{ 0, 0, 2,-2, 1, 0,  0, -3,  0,  3,  0,  0,  0,  0},    4,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0,  0, -3,  7,  0,  0,  0,  0,  2},    0,   -3,   -1,    0},
{{-2, 0, 0, 2, 0, 0,  4, -4,  0,  0,  0,  0,  0,  0},   -3,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -5,  8,  0,  0,  0,  0,  2},    5,    3,    1,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  5, -8,  0,  0,  0,  0,  0},    0,   11,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  0,  3,  0,  0,  0,  2},  118,    0,    0,  -52},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  0,  3,  0,  0,  0,  1},    0,   -5,   -3,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0, -3,  0,  0,  0,  0},  -28,   36,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  2, -4,  0,  0,  0,  0,  0,  0},    5,   -5,    0,    0},
{{ 0, 0, 0, 0, 0, 0, -2,  4,  0,  0,  0,  0,  0,  1},   14,  -59,  -31,   -8},
/* Planetary terms 341-350 */
{{ 0, 0, 1,-1, 1, 0, -2,  3,  0,  0,  0,  0,  0,  0},    0,    9,    5,    1},
{{ 0, 0, 0, 0, 0, 0, -2,  4,  0,  0,  0,  0,  0,  2}, -458,    0,    0,  198},
{{ 0, 0, 0, 0, 0, 0, -6,  9,  0,  0,  0,  0,  0,  2},    0,  -45,  -20,    0},
{{ 0, 0, 0, 0, 0, 0, -6,  9,  0,  0,  0,  0,  0,  1},    9,    0,    0,   -5},
{{ 0, 0, 0, 0, 0, 0,  6, -9,  0,  0,  0,  0,  0,  0},    0,   -3,    0,    0},
{{ 0, 0, 0, 0, 1, 0,  0,  1,  0, -2,  0,  0,  0,  0},    0,   -4,   -2,   -1},
{{ 0, 0, 2,-2, 1, 0, -2,  2,  0,  0,  0,  0,  0,  0},   11,    0,    0,   -6},
{{ 0, 0, 0, 0, 0, 0,  0, -4,  6,  0,  0,  0,  0,  2},    6,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  4, -6,  0,  0,  0,  0,  0},  -16,   23,    0,    0},
{{ 0, 0, 0, 0, 1, 0,  3, -4,  0,  0,  0,  0,  0,  0},    0,   -4,   -2,    0},
/* Planetary terms 351-360 */
{{ 0, 0, 0, 0, 0, 0,  0, -1,  0,  2,  0,  0,  0,  2},   -5,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0, -2,  0,  0,  0,  0}, -166,  269,    0,    0},
{{ 0, 0, 0, 0, 1, 0,  0,  1,  0, -1,  0,  0,  0,  0},   15,    0,    0,   -8},
{{ 0, 0, 0, 0, 0, 0, -5,  9,  0,  0,  0,  0,  0,  2},   10,    0,    0,   -4},
{{ 0, 0, 0, 0, 0, 0,  0,  3, -4,  0,  0,  0,  0,  0},  -78,   45,    0,    0},
{{ 0, 0, 0, 0, 0, 0, -3,  4,  0,  0,  0,  0,  0,  2},    0,   -5,   -2,    0},
{{ 0, 0, 0, 0, 0, 0, -3,  4,  0,  0,  0,  0,  0,  1},    7,    0,    0,   -4},
{{ 0, 0, 0, 0, 0, 0,  3, -4,  0,  0,  0,  0,  0,  0},   -5,  328,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  3, -4,  0,  0,  0,  0,  0,  1},    3,    0,    0,   -2},
{{ 0, 0, 0, 0, 1, 0,  0,  2, -2,  0,  0,  0,  0,  0},    5,    0,    0,   -2},
/* Planetary terms 361-370 */
{{ 0, 0, 0, 0, 1, 0,  0, -1,  0,  2,  0,  0,  0,  0},    0,    3,    1,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  0, -3,  0,  0,  0},   -3,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  1, -5,  0,  0,  0},   -3,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  0,  1,  0,  0,  0,  1},    0,   -4,   -2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0, -1,  0,  0,  0,  0},-1223,  -26,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0, -1,  0,  0,  0,  1},    0,    7,    3,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0, -3,  5,  0,  0,  0},    3,    0,    0,    0},
{{ 0, 0, 0, 0, 1, 0, -3,  4,  0,  0,  0,  0,  0,  0},    0,    3,    2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  0, -2,  0,  0,  0},   -6,   20,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  2, -2,  0,  0,  0,  0,  0}, -368,    0,    0,    0},
/* Planetary terms 371-380 */
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  0, -1,  0,  0,  0},  -75,    0,    0,    0},
{{ 0, 0, 0, 0, 1, 0,  0, -1,  0,  1,  0,  0,  0,  0},   11,    0,    0,   -6},
{{ 0, 0, 0, 0, 1, 0,  0, -2,  2,  0,  0,  0,  0,  0},    3,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0, -8, 14,  0,  0,  0,  0,  0,  2},   -3,    0,    0,    1},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  2, -5,  0,  0,  0},  -13,  -30,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  5, -8,  3,  0,  0,  0,  0},   21,    3,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  5, -8,  3,  0,  0,  0,  2},   -3,    0,    0,    1},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  0,  0,  0,  0,  0,  1},   -4,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  0,  0,  0,  0,  0},    8,  -27,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  3, -8,  3,  0,  0,  0,  0},  -19,  -11,    0,    0},
/* Planetary terms 381-390 */
{{ 0, 0, 0, 0, 0, 0,  0, -3,  8, -3,  0,  0,  0,  2},   -4,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0, -2,  5,  0,  0,  2},    0,    5,    2,    0},
{{ 0, 0, 0, 0, 0, 0, -8, 12,  0,  0,  0,  0,  0,  2},   -6,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0, -8, 12,  0,  0,  0,  0,  0,  0},   -8,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  1, -2,  0,  0,  0},   -1,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  0,  1,  0,  0,  2},  -14,    0,    0,    6},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  2,  0,  0,  0,  0,  0},    6,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  2,  0,  0,  0,  0,  2},  -74,    0,    0,   32},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  0,  2,  0,  0,  2},    0,   -3,   -1,    0},
{{ 0, 0, 2,-2, 1, 0, -5,  5,  0,  0,  0,  0,  0,  0},    4,    0,    0,   -2},
/* Planetary terms 391-400 */
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  1,  0,  0,  0,  0},    8,   11,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  1,  0,  0,  0,  1},    0,    3,    2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  1,  0,  0,  0,  2}, -262,    0,    0,  114},
{{ 0, 0, 0, 0, 0, 0,  3, -6,  0,  0,  0,  0,  0,  0},    0,   -4,    0,    0},
{{ 0, 0, 0, 0, 0, 0, -3,  6,  0,  0,  0,  0,  0,  1},   -7,    0,    0,    4},
{{ 0, 0, 0, 0, 0, 0, -3,  6,  0,  0,  0,  0,  0,  2},    0,  -27,  -12,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  4,  0,  0,  0,  0,  2},  -19,   -8,   -4,    8},
{{ 0, 0, 0, 0, 0, 0, -5,  7,  0,  0,  0,  0,  0,  2},  202,    0,    0,  -87},
{{ 0, 0, 0, 0, 0, 0, -5,  7,  0,  0,  0,  0,  0,  1},   -8,   35,   19,    5},
{{ 0, 0, 1,-1, 1, 0, -5,  6,  0,  0,  0,  0,  0,  0},    0,    4,    2,    0},
/* Planetary terms 401-410 */
{{ 0, 0, 0, 0, 0, 0,  5, -7,  0,  0,  0,  0,  0,  0},   16,   -5,    0,    0},
{{ 0, 0, 2,-2, 1, 0,  0, -1,  0,  1,  0,  0,  0,  0},    5,    0,    0,   -3},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  0,  1,  0,  0,  0,  0},    0,   -3,    0,    0},
{{ 0, 0, 0, 0, 0,-1,  0,  3,  0,  0,  0,  0,  0,  2},    1,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  2,  0,  0,  0,  2},  -35,  -48,  -21,   15},
{{ 0, 0, 0, 0, 0, 0,  0, -2,  6,  0,  0,  0,  0,  2},   -3,   -5,   -2,    1},
{{ 0, 0, 0, 0, 1, 0,  2, -2,  0,  0,  0,  0,  0,  0},    6,    0,    0,   -3},
{{ 0, 0, 0, 0, 0, 0,  0, -6,  9,  0,  0,  0,  0,  2},    3,    0,    0,   -1},
{{ 0, 0, 0, 0, 0, 0,  0,  6, -9,  0,  0,  0,  0,  0},    0,   -5,    0,    0},
{{ 0, 0, 0, 0, 0, 0, -2,  2,  0,  0,  0,  0,  0,  1},   12,   55,   29,   -6},
/* Planetary terms 411-420 */
{{ 0, 0, 1,-1, 1, 0, -2,  1,  0,  0,  0,  0,  0,  0},    0,    5,    3,    0},
{{ 0, 0, 0, 0, 0, 0,  2, -2,  0,  0,  0,  0,  0,  0}, -598,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  2, -2,  0,  0,  0,  0,  0,  1},   -3,  -13,   -7,    1},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  0,  3,  0,  0,  0,  2},   -5,   -7,   -3,    2},
{{ 0, 0, 0, 0, 0, 0,  0, -5,  7,  0,  0,  0,  0,  2},    3,    0,    0,   -1},
{{ 0, 0, 0, 0, 0, 0,  0,  5, -7,  0,  0,  0,  0,  0},    5,   -7,    0,    0},
{{ 0, 0, 0, 0, 1, 0, -2,  2,  0,  0,  0,  0,  0,  0},    4,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  4, -5,  0,  0,  0,  0,  0},   16,   -6,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  1, -3,  0,  0,  0,  0,  0,  0},    8,   -3,    0,    0},
{{ 0, 0, 0, 0, 0, 0, -1,  3,  0,  0,  0,  0,  0,  1},    8,  -31,  -16,   -4},
/* Planetary terms 421-430 */
{{ 0, 0, 1,-1, 1, 0, -1,  2,  0,  0,  0,  0,  0,  0},    0,    3,    1,    0},
{{ 0, 0, 0, 0, 0, 0, -1,  3,  0,  0,  0,  0,  0,  2},  113,    0,    0,  -49},
{{ 0, 0, 0, 0, 0, 0, -7, 10,  0,  0,  0,  0,  0,  2},    0,  -24,  -10,    0},
{{ 0, 0, 0, 0, 0, 0, -7, 10,  0,  0,  0,  0,  0,  1},    4,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  3, -3,  0,  0,  0,  0,  0},   27,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0, -4,  8,  0,  0,  0,  0,  0,  2},   -3,    0,    0,    1},
{{ 0, 0, 0, 0, 0, 0, -4,  5,  0,  0,  0,  0,  0,  2},    0,   -4,   -2,    0},
{{ 0, 0, 0, 0, 0, 0, -4,  5,  0,  0,  0,  0,  0,  1},    5,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0,  4, -5,  0,  0,  0,  0,  0,  0},    0,   -3,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  1,  0,  0,  0,  0,  2},  -13,    0,    0,    6},
/* Planetary terms 431-440 */
{{ 0, 0, 0, 0, 0, 0,  0, -2,  0,  5,  0,  0,  0,  2},    5,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  3,  0,  0,  0,  0,  2},  -18,  -10,   -4,    8},
{{ 0, 0, 0, 0, 0, 0,  1,  0,  0,  0,  0,  0,  0,  0},   -4,  -28,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  1,  0,  0,  0,  0,  0,  0,  2},   -5,    6,    3,    2},
{{ 0, 0, 0, 0, 0, 0, -9, 13,  0,  0,  0,  0,  0,  2},   -3,    0,    0,    1},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  5,  0,  0,  0,  0,  2},   -5,   -9,   -4,    2},
{{ 0, 0, 0, 0, 0, 0,  0, -2,  0,  4,  0,  0,  0,  2},   17,    0,    0,   -7},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0, -4,  0,  0,  0,  0},   11,    4,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -2,  7,  0,  0,  0,  0,  2},    0,   -6,   -2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0, -3,  0,  0,  0,  0},   83,   15,    0,    0},
/* Planetary terms 441-450 */
{{ 0, 0, 0, 0, 0, 0, -2,  5,  0,  0,  0,  0,  0,  1},   -4,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0, -2,  5,  0,  0,  0,  0,  0,  2},    0, -114,  -49,    0},
{{ 0, 0, 0, 0, 0, 0, -6,  8,  0,  0,  0,  0,  0,  2},  117,    0,    0,  -51},
{{ 0, 0, 0, 0, 0, 0, -6,  8,  0,  0,  0,  0,  0,  1},   -5,   19,   10,    2},
{{ 0, 0, 0, 0, 0, 0,  6, -8,  0,  0,  0,  0,  0,  0},   -3,    0,    0,    0},
{{ 0, 0, 0, 0, 1, 0,  0,  2,  0, -2,  0,  0,  0,  0},   -3,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0,  0, -3,  9,  0,  0,  0,  0,  2},    0,   -3,   -1,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  5, -6,  0,  0,  0,  0,  0},    3,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  5, -6,  0,  0,  0,  0,  2},    0,   -6,   -2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0, -2,  0,  0,  0,  0},  393,    3,    0,    0},
/* Planetary terms 451-460 */
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0, -2,  0,  0,  0,  1},   -4,   21,   11,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0, -2,  0,  0,  0,  2},   -6,    0,   -1,    3},
{{ 0, 0, 0, 0, 0, 0, -5, 10,  0,  0,  0,  0,  0,  2},   -3,    8,    4,    1},
{{ 0, 0, 0, 0, 0, 0,  0,  4, -4,  0,  0,  0,  0,  0},    8,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  4, -4,  0,  0,  0,  0,  2},   18,  -29,  -13,   -8},
{{ 0, 0, 0, 0, 0, 0, -3,  3,  0,  0,  0,  0,  0,  1},    8,   34,   18,   -4},
{{ 0, 0, 0, 0, 0, 0,  3, -3,  0,  0,  0,  0,  0,  0},   89,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  3, -3,  0,  0,  0,  0,  0,  1},    3,   12,    6,   -1},
{{ 0, 0, 0, 0, 0, 0,  3, -3,  0,  0,  0,  0,  0,  2},   54,  -15,   -7,  -24},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  0, -3,  0,  0,  0},    0,    3,    0,    0},
/* Planetary terms 461-470 */
{{ 0, 0, 0, 0, 0, 0,  0, -5, 13,  0,  0,  0,  0,  2},    3,    0,    0,   -1},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0, -1,  0,  0,  0,  0},    0,   35,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0, -1,  0,  0,  0,  2}, -154,  -30,  -13,   67},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  0, -2,  0,  0,  0},   15,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  0, -2,  0,  0,  1},    0,    4,    2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  3, -2,  0,  0,  0,  0,  0},    0,    9,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  3, -2,  0,  0,  0,  0,  2},   80,  -71,  -31,  -35},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  0, -1,  0,  0,  2},    0,  -20,   -9,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -6, 15,  0,  0,  0,  0,  2},   11,    5,    2,   -5},
{{ 0, 0, 0, 0, 0, 0, -8, 15,  0,  0,  0,  0,  0,  2},   61,  -96,  -42,  -27},
/* Planetary terms 471-480 */
{{ 0, 0, 0, 0, 0, 0, -3,  9, -4,  0,  0,  0,  0,  2},   14,    9,    4,   -6},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  2, -5,  0,  0,  2},  -11,   -6,   -3,    5},
{{ 0, 0, 0, 0, 0, 0,  0, -2,  8, -1, -5,  0,  0,  2},    0,   -3,   -1,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  6, -8,  3,  0,  0,  0,  2},  123, -415, -180,  -53},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  0,  0,  0,  0,  0},    0,    0,    0,  -35},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  0,  0,  0,  0,  0},   -5,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  0,  0,  0,  0,  1},    7,  -32,  -17,   -4},
{{ 0, 0, 1,-1, 1, 0,  0,  1,  0,  0,  0,  0,  0,  0},    0,   -9,   -5,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  0,  0,  0,  0,  1},    0,   -4,    2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  0,  0,  0,  0,  2},  -89,    0,    0,   38},
/* Planetary terms 481-490 */
{{ 0, 0, 0, 0, 0, 0,  0, -6, 16, -4, -5,  0,  0,  2},    0,  -86,  -19,   -6},
{{ 0, 0, 0, 0, 0, 0,  0, -2,  8, -3,  0,  0,  0,  2},    0,    0,  -19,    6},
{{ 0, 0, 0, 0, 0, 0,  0, -2,  8, -3,  0,  0,  0,  2}, -123, -416, -180,   53},
{{ 0, 0, 0, 0, 0, 0,  0,  6, -8,  1,  5,  0,  0,  2},    0,   -3,   -1,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0, -2,  5,  0,  0,  2},   12,   -6,   -3,   -5},
{{ 0, 0, 0, 0, 0, 0,  3, -5,  4,  0,  0,  0,  0,  2},  -13,    9,    4,    6},
{{ 0, 0, 0, 0, 0, 0, -8, 11,  0,  0,  0,  0,  0,  2},    0,  -15,   -7,    0},
{{ 0, 0, 0, 0, 0, 0, -8, 11,  0,  0,  0,  0,  0,  1},    3,    0,    0,   -1},
{{ 0, 0, 0, 0, 0, 0, -8, 11,  0,  0,  0,  0,  0,  2},  -62,  -97,  -42,   27},
{{ 0, 0, 0, 0, 0, 0,  0, 11,  0,  0,  0,  0,  0,  2},  -11,    5,    2,    5},
/* Planetary terms 491-500 */
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  0,  1,  0,  0,  2},    0,  -19,   -8,    0},
{{ 0, 0, 0, 0, 0, 0,  3, -3,  0,  2,  0,  0,  0,  2},   -3,    0,    0,    1},
{{ 0, 0, 2,-2, 1, 0,  0,  4, -8,  3,  0,  0,  0,  0},    0,    4,    2,    0},
{{ 0, 0, 1,-1, 0, 0,  0,  1,  0,  0,  0,  0,  0,  0},    0,    3,    0,    0},
{{ 0, 0, 2,-2, 1, 0,  0, -4,  8, -3,  0,  0,  0,  0},    0,    4,    2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  1,  2,  0,  0,  0,  0,  2},  -85,  -70,  -31,   37},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  1,  0,  0,  0,  2},  163,  -12,   -5,  -72},
{{ 0, 0, 0, 0, 0, 0, -3,  7,  0,  0,  0,  0,  0,  2},  -63,  -16,   -7,   28},
{{ 0, 0, 0, 0, 0, 0,  0,  0,  4,  0,  0,  0,  0,  2},  -21,  -32,  -14,    9},
{{ 0, 0, 0, 0, 0, 0, -5,  6,  0,  0,  0,  0,  0,  2},    0,   -3,   -1,    0},
/* Planetary terms 501-510 */
{{ 0, 0, 0, 0, 0, 0, -5,  6,  0,  0,  0,  0,  0,  1},    3,    0,    0,   -2},
{{ 0, 0, 0, 0, 0, 0,  5, -6,  0,  0,  0,  0,  0,  0},    0,    8,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  5, -6,  0,  0,  0,  0,  0,  2},    3,   10,    4,   -1},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0,  2,  0,  0,  0,  2},    3,    0,    0,   -1},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  6,  0,  0,  0,  0,  2},    0,   -7,   -3,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  7, -9,  0,  0,  0,  0,  2},    0,   -4,   -2,    0},
{{ 0, 0, 0, 0, 0, 0,  2, -1,  0,  0,  0,  0,  0,  0},    6,   19,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  2, -1,  0,  0,  0,  0,  0,  2},    5, -173,  -75,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  6, -7,  0,  0,  0,  0,  2},    0,   -7,   -3,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  5, -5,  0,  0,  0,  0,  2},    7,  -12,   -5,   -3},
/* Planetary terms 511-520 */
{{ 0, 0, 0, 0, 0, 0, -1,  4,  0,  0,  0,  0,  0,  1},   -3,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0, -1,  4,  0,  0,  0,  0,  0,  2},    3,   -4,   -2,   -1},
{{ 0, 0, 0, 0, 0, 0, -7,  9,  0,  0,  0,  0,  0,  2},   74,    0,    0,  -32},
{{ 0, 0, 0, 0, 0, 0, -7,  9,  0,  0,  0,  0,  0,  1},   -3,   12,    6,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  4, -3,  0,  0,  0,  0,  2},   26,  -14,   -6,  -11},
{{ 0, 0, 0, 0, 0, 0,  0,  3, -1,  0,  0,  0,  0,  2},   19,    0,    0,   -8},
{{ 0, 0, 0, 0, 0, 0, -4,  4,  0,  0,  0,  0,  0,  1},    6,   24,   13,   -3},
{{ 0, 0, 0, 0, 0, 0,  4, -4,  0,  0,  0,  0,  0,  0},   83,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  4, -4,  0,  0,  0,  0,  0,  1},    0,  -10,   -5,    0},
{{ 0, 0, 0, 0, 0, 0,  4, -4,  0,  0,  0,  0,  0,  2},   11,   -3,   -1,   -5},
/* Planetary terms 521-530 */
{{ 0, 0, 0, 0, 0, 0,  0,  2,  1,  0,  0,  0,  0,  2},    3,    0,    1,   -1},
{{ 0, 0, 0, 0, 0, 0,  0, -3,  0,  5,  0,  0,  0,  2},    3,    0,    0,   -1},
{{ 0, 0, 0, 0, 0, 0,  1,  1,  0,  0,  0,  0,  0,  0},   -4,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  1,  1,  0,  0,  0,  0,  0,  1},    5,  -23,  -12,   -3},
{{ 0, 0, 0, 0, 0, 0,  1,  1,  0,  0,  0,  0,  0,  2}, -339,    0,    0,  147},
{{ 0, 0, 0, 0, 0, 0, -9, 12,  0,  0,  0,  0,  0,  2},    0,  -10,   -5,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  3,  0, -4,  0,  0,  0,  0},    5,    0,    0,    0},
{{ 0, 0, 2,-2, 1, 0,  1, -1,  0,  0,  0,  0,  0,  0},    3,    0,    0,   -1},
{{ 0, 0, 0, 0, 0, 0,  0,  7, -8,  0,  0,  0,  0,  2},    0,   -4,   -2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  3,  0, -3,  0,  0,  0,  0},   18,   -3,    0,    0},
/* Planetary terms 531-540 */
{{ 0, 0, 0, 0, 0, 0,  0,  3,  0, -3,  0,  0,  0,  2},    9,  -11,   -5,   -4},
{{ 0, 0, 0, 0, 0, 0, -2,  6,  0,  0,  0,  0,  0,  2},   -8,    0,    0,    4},
{{ 0, 0, 0, 0, 0, 0, -6,  7,  0,  0,  0,  0,  0,  1},    3,    0,    0,   -1},
{{ 0, 0, 0, 0, 0, 0,  6, -7,  0,  0,  0,  0,  0,  0},    0,    9,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  6, -6,  0,  0,  0,  0,  2},    6,   -9,   -4,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  3,  0, -2,  0,  0,  0,  0},   -4,  -12,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  3,  0, -2,  0,  0,  0,  2},   67,  -91,  -39,  -29},
{{ 0, 0, 0, 0, 0, 0,  0,  5, -4,  0,  0,  0,  0,  2},   30,  -18,   -8,  -13},
{{ 0, 0, 0, 0, 0, 0,  3, -2,  0,  0,  0,  0,  0,  0},    0,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  3, -2,  0,  0,  0,  0,  0,  2},    0, -114,  -50,    0},
/* Planetary terms 541-550 */
{{ 0, 0, 0, 0, 0, 0,  0,  3,  0, -1,  0,  0,  0,  2},    0,    0,    0,   23},
{{ 0, 0, 0, 0, 0, 0,  0,  3,  0, -1,  0,  0,  0,  2},  517,   16,    7, -224},
{{ 0, 0, 0, 0, 0, 0,  0,  3,  0,  0, -2,  0,  0,  2},    0,   -7,   -3,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  4, -2,  0,  0,  0,  0,  2},  143,   -3,   -1,  -62},
{{ 0, 0, 0, 0, 0, 0,  0,  3,  0,  0, -1,  0,  0,  2},   29,    0,    0,  -13},
{{ 0, 0, 2,-2, 1, 0,  0,  1,  0, -1,  0,  0,  0,  0},   -4,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0, -8, 16,  0,  0,  0,  0,  0,  2},   -6,    0,    0,    3},
{{ 0, 0, 0, 0, 0, 0,  0,  3,  0,  2, -5,  0,  0,  2},    5,   12,    5,   -2},
{{ 0, 0, 0, 0, 0, 0,  0,  7, -8,  3,  0,  0,  0,  2},  -25,    0,    0,   11},
{{ 0, 0, 0, 0, 0, 0,  0, -5, 16, -4, -5,  0,  0,  2},   -3,    0,    0,    1},
/* Planetary terms 551-560 */
{{ 0, 0, 0, 0, 0, 0,  0,  3,  0,  0,  0,  0,  0,  2},    0,    4,    2,    0},
{{ 0, 0, 0, 0, 0, 0,  0, -1,  8, -3,  0,  0,  0,  2},  -22,   12,    5,   10},
{{ 0, 0, 0, 0, 0, 0, -8, 10,  0,  0,  0,  0,  0,  2},   50,    0,    0,  -22},
{{ 0, 0, 0, 0, 0, 0, -8, 10,  0,  0,  0,  0,  0,  1},    0,    7,    4,    0},
{{ 0, 0, 0, 0, 0, 0, -8, 10,  0,  0,  0,  0,  0,  2},    0,    3,    1,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  2,  0,  0,  0,  0,  2},   -4,    4,    2,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  3,  0,  1,  0,  0,  0,  2},   -5,  -11,   -5,    2},
{{ 0, 0, 0, 0, 0, 0, -3,  8,  0,  0,  0,  0,  0,  2},    0,    4,    2,    0},
{{ 0, 0, 0, 0, 0, 0, -5,  5,  0,  0,  0,  0,  0,  1},    4,   17,    9,   -2},
{{ 0, 0, 0, 0, 0, 0,  5, -5,  0,  0,  0,  0,  0,  0},   59,    0,    0,    0},
/* Planetary terms 561-570 */
{{ 0, 0, 0, 0, 0, 0,  5, -5,  0,  0,  0,  0,  0,  1},    0,   -4,   -2,    0},
{{ 0, 0, 0, 0, 0, 0,  5, -5,  0,  0,  0,  0,  0,  2},   -8,    0,    0,    4},
{{ 0, 0, 0, 0, 0, 0,  2,  0,  0,  0,  0,  0,  0,  0},   -3,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  2,  0,  0,  0,  0,  0,  0,  1},    4,  -15,   -8,   -2},
{{ 0, 0, 0, 0, 0, 0,  2,  0,  0,  0,  0,  0,  0,  2},  370,   -8,    0, -160},
{{ 0, 0, 0, 0, 0, 0,  0,  7, -7,  0,  0,  0,  0,  2},    0,    0,   -3,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  7, -7,  0,  0,  0,  0,  2},    0,    3,    1,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  6, -5,  0,  0,  0,  0,  2},   -6,    3,    1,    3},
{{ 0, 0, 0, 0, 0, 0,  7, -8,  0,  0,  0,  0,  0,  0},    0,    6,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  5, -3,  0,  0,  0,  0,  2},  -10,    0,    0,    4},
/* Planetary terms 571-580 */
{{ 0, 0, 0, 0, 0, 0,  4, -3,  0,  0,  0,  0,  0,  2},    0,    9,    4,    0},
{{ 0, 0, 0, 0, 0, 0,  1,  2,  0,  0,  0,  0,  0,  2},    4,   17,    7,   -2},
{{ 0, 0, 0, 0, 0, 0, -9, 11,  0,  0,  0,  0,  0,  2},   34,    0,    0,  -15},
{{ 0, 0, 0, 0, 0, 0, -9, 11,  0,  0,  0,  0,  0,  1},    0,    5,    3,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  4,  0, -4,  0,  0,  0,  2},   -5,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  4,  0, -3,  0,  0,  0,  2},  -37,   -7,   -3,   16},
{{ 0, 0, 0, 0, 0, 0, -6,  6,  0,  0,  0,  0,  0,  1},    3,   13,    7,   -2},
{{ 0, 0, 0, 0, 0, 0,  6, -6,  0,  0,  0,  0,  0,  0},   40,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  6, -6,  0,  0,  0,  0,  0,  1},    0,   -3,   -2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  4,  0, -2,  0,  0,  0,  2}, -184,   -3,   -1,   80},
/* Planetary terms 581-590 */
{{ 0, 0, 0, 0, 0, 0,  0,  6, -4,  0,  0,  0,  0,  2},   -3,    0,    0,    1},
{{ 0, 0, 0, 0, 0, 0,  3, -1,  0,  0,  0,  0,  0,  0},   -3,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  3, -1,  0,  0,  0,  0,  0,  1},    0,  -10,   -6,   -1},
{{ 0, 0, 0, 0, 0, 0,  3, -1,  0,  0,  0,  0,  0,  2},   31,   -6,    0,  -13},
{{ 0, 0, 0, 0, 0, 0,  0,  4,  0, -1,  0,  0,  0,  2},   -3,  -32,  -14,    1},
{{ 0, 0, 0, 0, 0, 0,  0,  4,  0,  0, -2,  0,  0,  2},   -7,    0,    0,    3},
{{ 0, 0, 0, 0, 0, 0,  0,  5, -2,  0,  0,  0,  0,  2},    0,   -8,   -4,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  4,  0,  0,  0,  0,  0,  0},    3,   -4,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  8, -9,  0,  0,  0,  0,  0,  0},    0,    4,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  5, -4,  0,  0,  0,  0,  0,  2},    0,    3,    1,    0},
/* Planetary terms 591-600 */
{{ 0, 0, 0, 0, 0, 0,  2,  1,  0,  0,  0,  0,  0,  2},   19,  -23,  -10,    2},
{{ 0, 0, 0, 0, 0, 0,  2,  1,  0,  0,  0,  0,  0,  1},    0,    0,    0,  -10},
{{ 0, 0, 0, 0, 0, 0,  2,  1,  0,  0,  0,  0,  0,  1},    0,    3,    2,    0},
{{ 0, 0, 0, 0, 0, 0, -7,  7,  0,  0,  0,  0,  0,  1},    0,    9,    5,   -1},
{{ 0, 0, 0, 0, 0, 0,  7, -7,  0,  0,  0,  0,  0,  0},   28,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  4, -2,  0,  0,  0,  0,  0,  1},    0,   -7,   -4,    0},
{{ 0, 0, 0, 0, 0, 0,  4, -2,  0,  0,  0,  0,  0,  2},    8,   -4,    0,   -4},
{{ 0, 0, 0, 0, 0, 0,  4, -2,  0,  0,  0,  0,  0,  0},    0,    0,   -2,    0},
{{ 0, 0, 0, 0, 0, 0,  4, -2,  0,  0,  0,  0,  0,  0},    0,    3,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  5,  0, -4,  0,  0,  0,  2},   -3,    0,    0,    1},
/* Planetary terms 601-610 */
{{ 0, 0, 0, 0, 0, 0,  0,  5,  0, -3,  0,  0,  0,  2},   -9,    0,    1,    4},
{{ 0, 0, 0, 0, 0, 0,  0,  5,  0, -2,  0,  0,  0,  2},    3,   12,    5,   -1},
{{ 0, 0, 0, 0, 0, 0,  3,  0,  0,  0,  0,  0,  0,  2},   17,   -3,   -1,    0},
{{ 0, 0, 0, 0, 0, 0, -8,  8,  0,  0,  0,  0,  0,  1},    0,    7,    4,    0},
{{ 0, 0, 0, 0, 0, 0,  8, -8,  0,  0,  0,  0,  0,  0},   19,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  5, -3,  0,  0,  0,  0,  0,  1},    0,   -5,   -3,    0},
{{ 0, 0, 0, 0, 0, 0,  5, -3,  0,  0,  0,  0,  0,  2},   14,   -3,    0,   -1},
{{ 0, 0, 0, 0, 0, 0, -9,  9,  0,  0,  0,  0,  0,  1},    0,    0,   -1,    0},
{{ 0, 0, 0, 0, 0, 0, -9,  9,  0,  0,  0,  0,  0,  1},    0,    0,    0,   -5},
{{ 0, 0, 0, 0, 0, 0, -9,  9,  0,  0,  0,  0,  0,  1},    0,    5,    3,    0},
/* Planetary terms 611-620 */
{{ 0, 0, 0, 0, 0, 0,  9, -9,  0,  0,  0,  0,  0,  0},   13,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  6, -4,  0,  0,  0,  0,  0,  1},    0,   -3,   -2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  6,  0,  0,  0,  0,  0,  2},    2,    9,    4,    3},
{{ 0, 0, 0, 0, 0, 0,  0,  6,  0,  0,  0,  0,  0,  0},    0,    0,    0,   -4},
{{ 0, 0, 0, 0, 0, 0,  0,  6,  0,  0,  0,  0,  0,  0},    8,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  6,  0,  0,  0,  0,  0,  1},    0,    4,    2,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  6,  0,  0,  0,  0,  0,  2},    6,    0,    0,   -3},
{{ 0, 0, 0, 0, 0, 0,  0,  6,  0,  0,  0,  0,  0,  0},    6,    0,    0,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  6,  0,  0,  0,  0,  0,  1},    0,    3,    1,    0},
{{ 0, 0, 0, 0, 0, 0,  0,  6,  0,  0,  0,  0,  0,  2},    5,    0,    0,   -2},
/* Planetary terms 621-630 */
{{ 0, 0, 0, 0, 0, 0,  0,  0,  0,  0,  0,  0,  0,  2},    3,    0,    0,   -1},
{{ 1, 0, 0,-2, 0, 0,  0,  2,  0, -2,  0,  0,  0,  0},   -3,    0,    0,    0},
{{ 1, 0, 0,-2, 0, 0,  2, -2,  0,  0,  0,  0,  0,  0},    6,    0,    0,    0},
{{ 1, 0, 0,-2, 0, 0,  0,  1,  0, -1,  0,  0,  0,  0},    7,    0,    0,    0},
{{ 1, 0, 0,-2, 0, 0,  1, -1,  0,  0,  0,  0,  0,  0},   -4,    0,    0,    0},
{{-1, 0, 0, 0, 0, 0,  3, -3,  0,  0,  0,  0,  0,  0},    4,    0,    0,    0},
{{-1, 0, 0, 0, 0, 0,  0,  2,  0, -2,  0,  0,  0,  0},    6,    0,    0,    0},
{{-1, 0, 0, 2, 0, 0,  0,  4, -8,  3,  0,  0,  0,  0},    0,   -4,    0,    0},
{{ 1, 0, 0,-2, 0, 0,  0,  4, -8,  3,  0,  0,  0,  0},    0,   -4,    0,    0},
{{-2, 0, 0, 2, 0, 0,  0,  4, -8,  3,  0,  0,  0,  0},    5,    0,    0,    0},
/* Planetary terms 631-640 */
{{-1, 0, 0, 0, 0, 0,  0,  2,  0, -3,  0,  0,  0,  0},   -3,    0,    0,    0},
{{-1, 0, 0, 0, 0, 0,  0,  1,  0, -1,  0,  0,  0,  0},    4,    0,    0,    0},
{{-1, 0, 0, 0, 0, 0,  1, -1,  0,  0,  0,  0,  0,  0},   -5,    0,    0,    0},
{{-1, 0, 0, 2, 0, 0,  2, -2,  0,  0,  0,  0,  0,  0},    4,    0,    0,    0},
{{ 1, 0,-1, 1, 0, 0,  0,  1,  0,  0,  0,  0,  0,  0},    0,    3,    0,    0},
{{-1, 0, 0, 2, 0, 0,  0,  2,  0, -3,  0,  0,  0,  0},   13,    0,    0,    0},
{{-2, 0, 0, 0, 0, 0,  0,  2,  0, -3,  0,  0,  0,  0},   21,   11,    0,    0},
{{ 1, 0, 0, 0, 0, 0,  0,  4, -8,  3,  0,  0,  0,  0},    0,   -5,    0,    0},
{{-1, 0, 1,-1, 1, 0,  0, -1,  0,  0,  0,  0,  0,  0},    0,   -5,   -2,    0},
{{ 1, 0, 1,-1, 1, 0,  0, -1,  0,  0,  0,  0,  0,  0},    0,    5,    3,    0},
/* Planetary terms 641-650 */
{{-1, 0, 0, 0, 0, 0,  0,  4, -8,  3,  0,  0,  0,  0},    0,   -5,    0,    0},
{{-1, 0, 0, 2, 1, 0,  0,  2,  0, -2,  0,  0,  0,  0},   -3,    0,    0,    2},
{{ 0, 0, 0, 0, 0, 0,  0,  2,  0, -2,  0,  0,  0,  0},   20,   10,    0,    0},
{{-1, 0, 0, 2, 0, 0,  0,  2,  0, -2,  0,  0,  0,  0},  -34,    0,    0,    0},
{{-1, 0, 0, 2, 0, 0,  3, -3,  0,  0,  0,  0,  0,  0},  -19,    0,    0,    0},
{{ 1, 0, 0,-2, 1, 0,  0, -2,  0,  2,  0,  0,  0,  0},    3,    0,    0,   -2},
{{ 1, 0, 2,-2, 2, 0, -3,  3,  0,  0,  0,  0,  0,  0},   -3,    0,    0,    1},
{{ 1, 0, 2,-2, 2, 0,  0, -2,  0,  2,  0,  0,  0,  0},   -6,    0,    0,    3},
{{ 1, 0, 0, 0, 0, 0,  1, -1,  0,  0,  0,  0,  0,  0},   -4,    0,    0,    0},
{{ 1, 0, 0, 0, 0, 0,  0,  1,  0, -1,  0,  0,  0,  0},    3,    0,    0,    0},
/* Planetary terms 651-660 */
{{ 0, 0, 0,-2, 0, 0,  2, -2,  0,  0,  0,  0,  0,  0},    3,    0,    0,    0},
{{ 0, 0, 0,-2, 0, 0,  0,  1,  0, -1,  0,  0,  0,  0},    4,    0,    0,    0},
{{ 0, 0, 2, 0, 2, 0, -2,  2,  0,  0,  0,  0,  0,  0},    3,    0,    0,   -1},
{{ 0, 0, 2, 0, 2, 0,  0, -1,  0,  1,  0,  0,  0,  0},    6,    0,    0,   -3},
{{ 0, 0, 2, 0, 2, 0, -1,  1,  0,  0,  0,  0,  0,  0},   -8,    0,    0,    3},
{{ 0, 0, 2, 0, 2, 0, -2,  3,  0,  0,  0,  0,  0,  0},    0,    3,    1,    0},
{{ 0, 0, 0, 2, 0, 0,  0,  2,  0, -2,  0,  0,  0,  0},   -3,    0,    0,    0},
{{ 0, 0, 1, 1, 2, 0,  0,  1,  0,  0,  0,  0,  0,  0},    0,   -3,   -2,    0},
{{ 1, 0, 2, 0, 2, 0,  0,  1,  0,  0,  0,  0,  0,  0},  126,  -63,  -27,  -55},
{{-1, 0, 2, 0, 2, 0, 10, -3,  0,  0,  0,  0,  0,  0},   -5,    0,    1,    2},
/* Planetary terms 661-670 */
{{ 0, 0, 1, 1, 1, 0,  0,  1,  0,  0,  0,  0,  0,  0},   -3,   28,   15,    2},
{{ 1, 0, 2, 0, 2, 0,  0,  1,  0,  0,  0,  0,  0,  0},    5,    0,    1,   -2},
{{ 0, 0, 2, 0, 2, 0,  0,  4, -8,  3,  0,  0,  0,  0},    0,    9,    4,    1},
{{ 0, 0, 2, 0, 2, 0,  0, -4,  8, -3,  0,  0,  0,  0},    0,    9,    4,   -1},
{{-1, 0, 2, 0, 2, 0,  0, -4,  8, -3,  0,  0,  0,  0}, -126,  -63,  -27,   55},
{{ 2, 0, 2,-2, 2, 0,  0, -2,  0,  3,  0,  0,  0,  0},    3,    0,    0,   -1},
{{ 1, 0, 2, 0, 1, 0,  0, -2,  0,  3,  0,  0,  0,  0},   21,  -11,   -6,  -11},
{{ 0, 0, 1, 1, 0, 0,  0,  1,  0,  0,  0,  0,  0,  0},    0,   -4,    0,    0},
{{-1, 0, 2, 0, 1, 0,  0,  1,  0,  0,  0,  0,  0,  0},  -21,  -11,   -6,   11},
{{-2, 0, 2, 2, 2, 0,  0,  2,  0, -2,  0,  0,  0,  0},   -3,    0,    0,    1},
/* Planetary terms 671-680 */
{{ 0, 0, 2, 0, 2, 0,  2, -3,  0,  0,  0,  0,  0,  0},    0,    3,    1,    0},
{{ 0, 0, 2, 0, 2, 0,  1, -1,  0,  0,  0,  0,  0,  0},    8,    0,    0,   -4},
{{ 0, 0, 2, 0, 2, 0,  0,  1,  0, -1,  0,  0,  0,  0},   -6,    0,    0,    3},
{{ 0, 0, 2, 0, 2, 0,  2, -2,  0,  0,  0,  0,  0,  0},   -3,    0,    0,    1},
{{-1, 0, 2, 2, 2, 0,  0, -1,  0,  1,  0,  0,  0,  0},    3,    0,    0,   -1},
{{ 1, 0, 2, 0, 2, 0, -1,  1,  0,  0,  0,  0,  0,  0},   -3,    0,    0,    1},
{{-1, 0, 2, 2, 2, 0,  0,  2,  0, -3,  0,  0,  0,  0},   -5,    0,    0,    2},
{{ 2, 0, 2, 0, 2, 0,  0,  2,  0, -3,  0,  0,  0,  0},   24,  -12,   -5,  -11},
{{ 1, 0, 2, 0, 2, 0,  0, -4,  8, -3,  0,  0,  0,  0},    0,    3,    1,    0},
{{ 1, 0, 2, 0, 2, 0,  0,  4, -8,  3,  0,  0,  0,  0},    0,    3,    1,    0},
/* Planetary terms 681-687 */
{{ 1, 0, 1, 1, 1, 0,  0,  1,  0,  0,  0,  0,  0,  0},    0,    3,    2,    0},
{{ 0, 0, 2, 0, 2, 0,  0,  1,  0,  0,  0,  0,  0,  0},  -24,  -12,   -5,   10},
{{ 2, 0, 2, 0, 1, 0,  0,  1,  0,  0,  0,  0,  0,  0},    4,    0,   -1,   -2},
{{-1, 0, 2, 2, 2, 0,  0,  2,  0, -2,  0,  0,  0,  0},   13,    0,    0,   -6},
{{-1, 0, 2, 2, 2, 0,  3, -3,  0,  0,  0,  0,  0,  0},    7,    0,    0,   -3},
{{ 1, 0, 2, 0, 2, 0,  1, -1,  0,  0,  0,  0,  0,  0},    3,    0,    0,   -1},
{{ 0, 0, 2, 2, 2, 0,  0,  2,  0, -2,  0,  0,  0,  0},    3,    0,    0,   -1}
   };

#define NPL ( sizeof serpl / sizeof serpl[0] )

/* Miscellaneous */
   int i, j;
   double t, dp, de, a, sa, ca;



/* Interval between fundamental date J2000.0 and given date (JC). */
   t = ( date - DJM0 ) / DJC;

/* Initialize the nutation values. */
   dp = 0.0;
   de = 0.0;

/*
** ------------------
** PLANETARY NUTATION
** ------------------
**
** n.b. The MHB_2000 computes the luni-solar and planetary nutation in
**      different functions, using slightly different Delaunay arguments
**      in the two cases.  This behaviour is faithfully reproduced
**      here.  Use of the Simon et al. expressions for both cases leads
**      to negligible changes, well below 0.1 microarcsecond.
*/

/* Mean anomaly of the Moon. */
   fa[0] = fmod ( 2.35555598 + 8328.6914269554 * t, D2PI );

/* Mean anomaly of the Sun (in the event not needed). */
   fa[1] = fmod ( 6.24006013 + 628.301955 * t, D2PI );

/* Mean argument of the latitude of the Moon. */
   fa[2] = fmod ( 1.627905234 + 8433.466158131 * t, D2PI );

/* Mean elongation of the Moon from the Sun. */
   fa[3] = fmod ( 5.198466741 + 7771.3771468121 * t, D2PI );

/* Mean longitude of the ascending node of the Moon. */
   fa[4] = fmod ( 2.18243920 - 33.757045 * t, D2PI );

/* Planetary longitudes, Mercury through Neptune (Souchay et al. 1999). */
   fa[5] = fmod ( 4.402608842 + 2608.7903141574 * t, D2PI );
   fa[6] = fmod ( 3.176146697 + 1021.3285546211 * t, D2PI );
   fa[7] = fmod ( 1.753470314 +  628.3075849991 * t, D2PI );
   fa[8] = fmod ( 6.203480913 +  334.0612426700 * t, D2PI );
   fa[9] = fmod ( 0.599546497 +   52.9690962641 * t, D2PI );
   fa[10] = fmod ( 0.874016757 +   21.3299104960 * t, D2PI );
   fa[11] = fmod ( 5.481293871 +    7.4781598567 * t, D2PI );
   fa[12] = fmod ( 5.321159000 +    3.8127774000 * t, D2PI );

/* General accumulated precession in longitude. */
   fa[13] = ( 0.02438175 + 0.00000538691 * t ) * t;

/* Summation of planetary nutation series (in reverse order). */
   for ( i = NPL-1; i >= 0; i-- ) {

   /* Argument and functions. */
      a = 0.0;
      for ( j = 0; j < 14; j++ ) {
         a += ((double) serpl[i].nfa[j])*fa[j];
      }
      sa = sin( a );
      ca = cos( a );

   /* Add the term. */
      dp += ((double) serpl[i].ips)*sa + ((double) serpl[i].ipc)*ca;
      de += ((double) serpl[i].ies)*sa + ((double) serpl[i].iec)*ca;
   }

/* To microarcseconds. */
   dp /= 10.0;
   de /= 10.0;

/*
** -------------------
** LUNI-SOLAR NUTATION
** -------------------
**
** Fundamental (Delaunay) arguments from Simon et al. (1994).
*/

/* Mean anomaly of the Moon. */
   fa[0] = fmod (            485868.249036 +
                  t * (  1717915923.2178 +
                  t * (          31.8792 +
                  t * (           0.051635 +
                  t * (         - 0.00024470 ) ) ) ), TURNAS  ) * DAS2R;

/* Mean anomaly of the Sun. */
   fa[1] = fmod (         1287104.79305 +
                  t * (   129596581.0481 +
                  t * (         - 0.5532 +
                  t * (           0.000136 +
                  t * (         - 0.00001149 ) ) ) ), TURNAS  ) * DAS2R;

/* Mean argument of the latitude of the Moon. */
   fa[2] = fmod (            335779.526232 +
                  t * (  1739527262.8478 +
                  t * (        - 12.7512 +
                  t * (         - 0.001037 +
                  t * (           0.00000417 ) ) ) ), TURNAS  ) * DAS2R;

/* Mean elongation of the Moon from the Sun. */
   fa[3] = fmod (           1072260.70369 +
                  t * (  1602961601.2090 +
                  t * (         - 6.3706 +
                  t * (           0.006593 +
                  t * (         - 0.00003169 ) ) ) ), TURNAS ) * DAS2R;

/* Mean longitude of the ascending node of the Moon. */
   fa[4] = fmod (            450160.398036 +
                  t * (   - 6962890.5431 +
                  t * (           7.4722 +
                  t * (           0.007702 +
                  t * (         - 0.00005939 ) ) ) ), TURNAS ) * DAS2R;

/* Summation of luni-solar nutation series (in reverse order). */
   for ( i = NLS-1; i >= 0; i-- ) {

   /* Argument and functions. */
      a = 0.0;
      for ( j = 0; j < 5; j++ ) {
         a += ((double) serls[i].nfa[j])*fa[j];
      }
      sa = sin( a );
      ca = cos( a );

   /* Add the term. */
      dp += ( serls[i].ps + t*serls[i].pst )*sa + serls[i].pc*ca;
      de += ( serls[i].ec + t*serls[i].ect )*ca + serls[i].es*sa;
   }

/* To arcseconds. */
   dp /= 1e6;
   de /= 1e6;

/* ------- */
/* RESULTS */
/* ------- */

/* To radians. */
   *dpsi = dp * DAS2R;
   *deps = de * DAS2R;

}
